from serde import serde
from serde.json import from_json


class MyException(Exception):
    pass


@serde
class Foo:
    v: int

    def __post_init__(self) -> None:
        if self.v == 10:
            raise MyException("Invalid value")


def main() -> None:
    try:
        s = '{"v": 10}'
        print(f"From Json: {from_json(Foo, s)}")

    except MyException as e:
        # Any exception from __post_init__ won't be wrapped by SerdeError.
        print(f"Got user exception: {e}")


if __name__ == "__main__":
    main()



from plum import dispatch
from datetime import datetime
from serde import serde, add_serializer, add_deserializer
from serde.json import from_json, to_json
from typing import Type, Any


class MySerializer:
    @dispatch
    def serialize(self, value: datetime) -> str:
        return value.strftime("%d/%m/%y")


class MyDeserializer:
    @dispatch
    def deserialize(self, cls: Type[datetime], value: Any) -> datetime:
        return datetime.strptime(value, "%d/%m/%y")


class MySerializer2:
    @dispatch
    def serialize(self, value: int) -> str:
        return str(value)


class MyDeserializer2:
    @dispatch
    def deserialize(self, cls: Type[int], value: Any) -> int:
        return int(value)


class MySerializer3:
    @dispatch
    def serialize(self, value: float) -> str:
        return str(value)


class MyDeserializer3:
    @dispatch
    def deserialize(self, cls: Type[float], value: Any) -> float:
        return float(value)


add_serializer(MySerializer())
add_serializer(MySerializer2())
add_deserializer(MyDeserializer())
add_deserializer(MyDeserializer2())


@serde(class_serializer=MySerializer3(), class_deserializer=MyDeserializer3())
class Foo:
    a: datetime
    b: int
    c: float


def main() -> None:
    dt = datetime(2021, 1, 1, 0, 0, 0)
    f = Foo(dt, 10, 100.0)
    print(f"Into Json: {to_json(f)}")

    s = '{"a": "01/01/21", "b": "10", "c": "100.0"}'
    print(f"From Json: {from_json(Foo, s)}")


if __name__ == "__main__":
    main()



"""
bench.py - Benchmarking pyserde.
"""

from __future__ import annotations
import json
import timeit
from dataclasses import dataclass, field
from functools import partial
from pathlib import Path
from platform import python_implementation
from collections.abc import Callable
from typing import Any, Optional, Union

import click
import data
import dataclasses_class as dc
import pyserde_class as ps  # noqa: F401
import pyserde_nt_class as pn  # noqa: F401
import raw  # noqa: F401
from runner import Size

try:
    if python_implementation() != "PyPy":
        import attrs_class as at  # noqa: F401
        import cattrs_class as ca  # noqa: F401
        import dacite_class as da  # noqa: F401
        import marshmallow_class as ms  # noqa: F401
        import mashumaro_class as mc  # noqa: F401
        import pydantic_class as pd  # noqa: F401
except ImportError:
    pass


@dataclass
class Opt:
    """
    Benchmark options.
    """

    full: bool
    chart: bool
    output: Path

    def __post_init__(self) -> None:
        if not self.output.exists():
            self.output.mkdir()


@dataclass
class Bencher:
    name: str
    opt: Opt
    number: int = 10000
    repeat: int = 5
    result: list[tuple[str, float]] = field(default_factory=list)

    def run(
        self,
        name: str,
        func: Optional[Callable[..., Any]],
        expected: Optional[Union[Any, Callable[[Any], bool]]] = None,
        **kwargs: Any,
    ) -> None:
        """
        Run benchmark.
        """
        if not func:
            return

        f: Callable[[], Any]
        if kwargs:
            f = partial(func, **kwargs)
        else:
            f = func

        # Evaluate result only once.
        if expected:
            actual = f()
            if callable(expected):
                expected(actual)
            else:
                assert actual == expected, f"Expected: {expected}, Actual: {actual}"

        times = timeit.repeat(f, number=self.number, repeat=self.repeat)
        self.result.append((name, sum(times) / len(times)))
        times_str = ", ".join([f"{t:.6f}" for t in times])
        click.echo(f"{name:40s}\t{times_str}")
        self.draw_chart()

    def draw_chart(self) -> None:
        if self.opt.chart:
            try:
                import matplotlib.pyplot as plt
                import numpy as np
                import seaborn as sns  # type: ignore[import]

                x = np.array([r[0] for r in self.result])
                y = np.array([r[1] for r in self.result])
                chart = sns.barplot(x=x, y=y, palette="rocket")
                chart.set(ylabel=f"Elapsed time for {self.number} requests [sec]")
                for p in chart.patches:
                    chart.annotate(
                        format(p.get_height(), ".4f"),
                        (p.get_x() + p.get_width() / 2.0, p.get_height()),
                        ha="center",
                        va="center",
                        xytext=(0, 10),
                        textcoords="offset points",
                    )
                plt.xticks(rotation=20)
                plt.savefig(str(self.opt.output / f"{self.name}.png"))
                plt.close()
            except ImportError:
                pass


runners_base = ("raw", "dc", "ps", "pn")

runners_extra = ("da", "mc", "ms", "at", "ca", "pd")


def run(opt: Opt, name: str, tc: TestCase) -> None:
    """
    Run benchmark.
    """
    bench = Bencher(f"{name}-{tc.size.name.lower()}", opt, tc.number)
    click.echo(f"--- {bench.name} ---")
    runners = runners_base + runners_extra if opt.full else runners_base
    for runner_name in runners:
        runner = globals()[runner_name].new(tc.size)
        bench.run(runner.name, getattr(runner, name), tc.expected)


@dataclass
class TestCase:
    size: Size
    expected: Union[Any, Callable[[Any], bool]]
    number: int

    @classmethod
    def make(cls, size: Size, expected: Any = None, number: int = 10000) -> dict[Size, TestCase]:
        return {size: TestCase(size, expected, number)}


def equals_small(x: Any) -> None:
    y = dc.SMALL
    assert x.i == y.i and x.s == y.s and x.f == y.f and x.b == y.b, f"Expected: {x}, Actual: {y}"


def equals_medium(x: Any) -> None:
    y = dc.MEDIUM
    for xs, ys in zip(x.inner, y.inner, strict=True):
        assert (
            xs.i == xs.i and xs.s == ys.s and xs.f == ys.f and xs.b == ys.b
        ), f"Expected: {x}, Actual: {y}"


def equals_large(x: Any) -> None:
    """Validate large deserialized data structure"""
    # Use different reference based on the library type
    if hasattr(type(x), "model_fields"):  # Pydantic model
        try:
            import pydantic_class as pd

            y = pd.LARGE
        except ImportError:
            y = ps.LARGE
    else:
        y = ps.LARGE

    assert (
        x.customer_id == y.customer_id
    ), f"Customer ID mismatch: {x.customer_id} != {y.customer_id}"
    assert x.name == y.name, f"Name mismatch: {x.name} != {y.name}"
    assert x.email == y.email, f"Email mismatch: {x.email} != {y.email}"
    assert len(x.items_list) == len(
        y.items_list
    ), f"Items list length mismatch: {len(x.items_list)} != {len(y.items_list)}"
    assert len(x.nested_data) == len(
        y.nested_data
    ), f"Nested data keys mismatch: {len(x.nested_data)} != {len(y.nested_data)}"
    assert (
        x.loyalty_points == y.loyalty_points
    ), f"Loyalty points mismatch: {x.loyalty_points} != {y.loyalty_points}"
    assert x.created_at == y.created_at, f"Created at mismatch: {x.created_at} != {y.created_at}"

    # Validate preferences
    assert x.preferences["theme"] == y.preferences["theme"], "Theme preference mismatch"
    assert (
        x.preferences["notifications"] == y.preferences["notifications"]
    ), "Notifications preference mismatch"

    # Validate nested data structure
    assert "category_1" in x.nested_data, "category_1 not found in nested_data"
    assert (
        len(x.nested_data["category_1"]) == 50
    ), f"category_1 length mismatch: {len(x.nested_data['category_1'])}"


TESTCASES = {
    "se": {
        **TestCase.make(Size.Small, lambda x: json.loads(x) == json.loads(data.SMALL)),
        **TestCase.make(
            Size.Medium, lambda x: json.loads(x) == json.loads(data.MEDIUM), number=500
        ),
        **TestCase.make(Size.Large, lambda x: json.loads(x) == json.loads(data.LARGE), number=100),
    },
    "de": {
        **TestCase.make(Size.Small, equals_small),
        **TestCase.make(Size.Medium, equals_medium, number=500),
        **TestCase.make(Size.Large, equals_large, number=100),
    },
    "astuple": {
        **TestCase.make(Size.Small, data.SMALL_TUPLE),
        **TestCase.make(Size.Medium, number=500),
        **TestCase.make(Size.Large, number=100),
    },
    "asdict": {
        **TestCase.make(Size.Small, data.SMALL_DICT),
        **TestCase.make(Size.Medium, number=500),
        **TestCase.make(Size.Large, number=100),
    },
}


@click.command()
@click.option(
    "-f", "--full", type=bool, is_flag=True, default=False, help="Run full benchmark tests."
)
@click.option("-t", "--test", default="", help="Run specified test case only.")
@click.option(
    "-c",
    "--chart",
    type=bool,
    is_flag=True,
    default=False,
    help="Draw barcharts of benchmark results.",
)
@click.option(
    "-o",
    "--output",
    default="charts",
    callback=lambda _, __, p: Path(p),
    help="Output directory for charts.",
)
def main(full: bool, test: str, chart: bool, output: Path) -> None:
    """
    bench.py - Benchmarking pyserde and other libraries.
    """
    opt = Opt(full, chart, output)
    tests = (test,) if test else TESTCASES
    for test in tests:
        for size in Size:
            try:
                tc = TESTCASES[test][size]
                run(opt, test, tc)
            except KeyError:
                pass


if __name__ == "__main__":
    main()  # pyright: ignore[reportCallIssue]



from serde.json import from_json, to_json
from serde import field, serde


class Id(str):
    def __str__(self) -> str:
        return "ID " + self


@serde
class Foo:
    a: Id = field(default_factory=Id)
    b: dict[Id, float] = field(default_factory=dict)
    c: list[Id] = field(default_factory=list)


def main() -> None:
    f = Foo(Id("a"), {Id("b"): 1.0}, [Id("c")])
    print(f)
    print(type(f.a))
    print(type(list(f.b.keys())[0]))
    print(type(f.c[0]))

    d = to_json(f)
    print(d)

    ff = from_json(Foo, d)
    print(ff)
    print(type(ff.a))
    print(type(list(ff.b.keys())[0]))
    print(type(ff.c[0]))


if __name__ == "__main__":
    main()



import logging
import sys
import uuid
from dataclasses import dataclass
from ipaddress import IPv4Address
from typing import (
    Generic,
    NewType,
    Optional,
    TypeVar,
    Union,
    Any,
    Literal,
)
from uuid import UUID

import pytest

from serde import (
    SerdeError,
    from_dict,
    from_tuple,
    init as serde_init,
    logger,
    serde,
    to_dict,
    to_tuple,
    InternalTagging,
    AdjacentTagging,
    Untagged,
)
from serde.json import from_json, to_json

logging.basicConfig(level=logging.WARNING)
logger.setLevel(logging.DEBUG)

serde_init(True)


if sys.version_info[:3] >= (3, 10, 0):

    @serde
    @dataclass(unsafe_hash=True)
    class PriUnion:  # pyright: ignore[reportRedeclaration] # Version-specific redefinition
        """
        Union Primitives.
        """

        v: int | str | float | bool

else:

    @serde
    @dataclass(unsafe_hash=True)
    class PriUnion:  # type: ignore[no-redef] # pyright: ignore[reportRedeclaration] # Version-specific redefinition
        """
        Union Primitives.
        """

        v: Union[int, str, float, bool]


@serde
@dataclass(unsafe_hash=True)
class PriOptUnion:
    """
    Union Primitives.
    """

    v: Union[Optional[int], Optional[str], Optional[float], Optional[bool]]


@serde
@dataclass(unsafe_hash=True)
class OptionalUnion:
    """
    Union Primitives.
    """

    v: Optional[Union[int, str]]


@serde
@dataclass(unsafe_hash=True)
class ContUnion:
    """
    Union Containers.
    """

    v: Union[dict[str, int], list[int], list[str]]


@serde
@dataclass(unsafe_hash=True)
class LitUnion:
    """
    Union of literals
    """

    v: Union[int, Literal["foo", "bar", "*"]]  # noqa


def test_union() -> None:
    v = PriUnion(10)
    s = '{"v":10}'
    assert s == to_json(v)
    assert v == from_json(PriUnion, s)

    v = PriUnion(10.0)
    s = '{"v":10.0}'
    assert s == to_json(v)
    assert v == from_json(PriUnion, s)

    v = PriUnion("foo")
    s = '{"v":"foo"}'
    assert s == to_json(v)
    assert v == from_json(PriUnion, s)

    v = PriUnion(True)
    s = '{"v":true}'
    assert s == to_json(v)
    assert v == from_json(PriUnion, s)


def test_union_optional() -> None:
    v = PriOptUnion(10)
    s = '{"v":10}'
    assert s == to_json(v)
    assert v == from_json(PriOptUnion, s)

    v = PriOptUnion(None)
    s = '{"v":null}'
    assert s == to_json(v)
    assert v == from_json(PriOptUnion, s)

    v = PriOptUnion("foo")
    s = '{"v":"foo"}'
    assert s == to_json(v)
    assert v == from_json(PriOptUnion, s)

    v = PriOptUnion(10.0)
    s = '{"v":10.0}'
    assert s == to_json(v)
    assert v == from_json(PriOptUnion, s)

    v = PriOptUnion(False)
    s = '{"v":false}'
    assert s == to_json(v)
    assert v == from_json(PriOptUnion, s)

    assert PriOptUnion(None) == from_json(PriOptUnion, "{}")
    assert OptionalUnion(None) == from_json(OptionalUnion, "{}")


def test_union_containers() -> None:
    v = ContUnion([1, 2, 3])
    s = '{"v":[1,2,3]}'
    assert s == to_json(v)
    assert v == from_json(ContUnion, s)

    v = ContUnion(["1", "2", "3"])
    s = '{"v":["1","2","3"]}'
    assert s == to_json(v)
    assert v == from_json(ContUnion, s)

    v = ContUnion({"a": 1, "b": 2, "c": 3})
    s = '{"v":{"a":1,"b":2,"c":3}}'
    assert s == to_json(v)
    # Note: this only works because dict[str, int] comes first in Union otherwise a list would win
    assert v == from_json(ContUnion, s)


def test_union_with_literal() -> None:
    v = LitUnion(10)
    s = '{"v":10}'
    assert s == to_json(v)
    assert v == from_json(LitUnion, s)

    v = LitUnion("foo")
    s = '{"v":"foo"}'
    assert s == to_json(v)
    assert v == from_json(LitUnion, s)

    v = LitUnion("bar")
    s = '{"v":"bar"}'
    assert s == to_json(v)
    assert v == from_json(LitUnion, s)

    v = LitUnion("*")
    s = '{"v":"*"}'
    assert s == to_json(v)
    assert v == from_json(LitUnion, s)

    s = '{"v":"boo"}'

    with pytest.raises(SerdeError):
        from_json(LitUnion, s)


def test_union_with_complex_types() -> None:
    @serde
    class A:
        v: Union[int, IPv4Address, UUID]

    a_int = A(1)
    a_int_json = '{"v":1}'
    assert to_json(a_int) == a_int_json
    assert from_json(A, a_int_json) == a_int
    assert a_int == from_dict(A, to_dict(a_int))

    a_ip = A(IPv4Address("127.0.0.1"))
    a_ip_json = '{"v":"127.0.0.1"}'
    assert to_json(a_ip) == a_ip_json
    assert from_json(A, a_ip_json) == a_ip
    assert a_ip == from_dict(A, to_dict(a_ip))

    a_uid = A(UUID("a317958e-4cbb-4213-9f23-eaff1563c472"))
    a_uid_json = '{"v":"a317958e-4cbb-4213-9f23-eaff1563c472"}'
    assert to_json(a_uid) == a_uid_json
    assert from_json(A, a_uid_json) == a_uid
    assert a_uid == from_dict(A, to_dict(a_uid))


def test_union_with_complex_types_and_reuse_instances() -> None:
    @serde(reuse_instances_default=True)
    class A:
        v: Union[int, IPv4Address, UUID]

    a_int = A(1)
    a_int_roundtrip = from_dict(A, to_dict(a_int))
    assert a_int == a_int_roundtrip
    assert a_int.v is a_int_roundtrip.v

    a_ip = A(IPv4Address("127.0.0.1"))
    a_ip_roundtrip = from_dict(A, to_dict(a_ip))
    assert a_ip == a_ip_roundtrip
    assert a_ip.v is a_ip_roundtrip.v

    a_uid = A(UUID("a317958e-4cbb-4213-9f23-eaff1563c472"))
    a_uid_roundtrip = from_dict(A, to_dict(a_uid))
    assert a_uid == a_uid_roundtrip
    assert a_uid.v is a_uid_roundtrip.v


def test_optional_union_with_complex_types() -> None:
    @serde
    class A:
        v: Optional[Union[int, IPv4Address, UUID]]

    a = A(123)
    assert a == from_dict(A, to_dict(a, reuse_instances=False), reuse_instances=False)
    assert a == from_dict(A, to_dict(a, reuse_instances=True), reuse_instances=True)

    a_none = A(None)
    assert a_none == from_dict(A, to_dict(a_none, reuse_instances=False), reuse_instances=False)
    assert a_none == from_dict(A, to_dict(a_none, reuse_instances=True), reuse_instances=True)


def test_optional_complex_type_with_default() -> None:
    for T, default in [
        (IPv4Address, IPv4Address("127.0.0.1")),
        (UUID, UUID("9c244009-c60d-452b-a378-b8afdc0c2d90")),
    ]:

        @serde
        class A:
            id: Optional[T] = None  # type: ignore[valid-type]

        a = A(default)
        assert a == from_dict(A, to_dict(a, reuse_instances=False), reuse_instances=False)
        assert a == from_dict(A, to_dict(a, reuse_instances=True), reuse_instances=True)

        a_none = A(None)
        assert a_none == from_dict(A, to_dict(a_none, reuse_instances=False), reuse_instances=False)
        assert a_none == from_dict(A, to_dict(a_none, reuse_instances=True), reuse_instances=True)

        a_default = A()
        assert a_default == from_dict(
            A, to_dict(a_default, reuse_instances=False), reuse_instances=False
        )
        assert a_default == from_dict(
            A, to_dict(a_default, reuse_instances=True), reuse_instances=True
        )


def test_union_with_complex_types_in_containers() -> None:
    @serde
    class A:
        v: Union[list[IPv4Address], list[UUID]]

    a_ips = A([IPv4Address("127.0.0.1"), IPv4Address("10.0.0.1")])
    assert a_ips == from_dict(A, to_dict(a_ips, reuse_instances=False), reuse_instances=False)
    assert a_ips == from_dict(A, to_dict(a_ips, reuse_instances=True), reuse_instances=True)

    a_uids = A(
        [UUID("9c244009-c60d-452b-a378-b8afdc0c2d90"), UUID("5831dc09-20fe-4433-b476-5866b7143364")]
    )
    assert a_uids == from_dict(A, to_dict(a_uids, reuse_instances=False), reuse_instances=False)
    assert a_uids == from_dict(A, to_dict(a_uids, reuse_instances=True), reuse_instances=True)

    a_empty = A([])
    assert a_empty == from_dict(A, to_dict(a_empty, reuse_instances=False), reuse_instances=False)
    assert a_empty == from_dict(A, to_dict(a_empty, reuse_instances=True), reuse_instances=True)


def test_union_exception_if_nothing_matches() -> None:
    @serde
    class A:
        v: Union[IPv4Address, UUID]

    with pytest.raises(SerdeError) as ex1:
        from_dict(A, {"v": "not-ip-or-uuid"})
    assert str(ex1.value) == (
        "Can not deserialize 'not-ip-or-uuid' of type str into Union[IPv4Address, UUID].\n"
        "Reasons:\n"
        " Failed to deserialize into IPv4Address: Expected 4 octets in 'not-ip-or-uuid'\n"
        " Failed to deserialize into UUID: badly formed hexadecimal UUID string"
    )

    with pytest.raises(SerdeError) as ex2:
        from_dict(A, {"v": "not-ip-or-uuid"}, reuse_instances=True)
    assert str(ex2.value) == (
        "Can not deserialize 'not-ip-or-uuid' of type str into Union[IPv4Address, UUID].\n"
        "Reasons:\n"
        " Failed to deserialize into IPv4Address: Expected 4 octets in 'not-ip-or-uuid'\n"
        " Failed to deserialize into UUID: badly formed hexadecimal UUID string"
    )

    with pytest.raises(SerdeError) as ex3:
        from_dict(A, {"v": None})
    # omit reason because it is not the same for all python versions & operating systems
    assert str(ex3.value).startswith(
        "Can not deserialize None of type NoneType into Union[IPv4Address, UUID]."
    )

    with pytest.raises(SerdeError) as ex4:
        a = A(uuid.uuid4())
        a.v = "not-ip-or-uuid"  # type: ignore
        to_dict(a)
    assert (
        str(ex4.value)
        == "Can not serialize 'not-ip-or-uuid' of type str for Union[IPv4Address, UUID]"
    )

    with pytest.raises(SerdeError) as ex5:
        a = A(uuid.uuid4())
        a.v = "not-ip-or-uuid"  # type: ignore
        to_dict(a, reuse_instances=True)
    assert (
        str(ex5.value)
        == "Can not serialize 'not-ip-or-uuid' of type str for Union[IPv4Address, UUID]"
    )

    with pytest.raises(SerdeError) as ex6:
        a = A(uuid.uuid4())
        a.v = None  # type: ignore
        to_dict(a, reuse_instances=True)
    assert str(ex6.value) == "Can not serialize None of type NoneType for Union[IPv4Address, UUID]"


def test_union_in_union() -> None:
    @serde
    class A:
        v: Union[UUID, Union[int, str]]

    a_uuid = A(UUID("00611ee9-7ca3-41d3-9607-ea7268e264ea"))
    assert a_uuid == from_dict(A, to_dict(a_uuid, reuse_instances=False), reuse_instances=False)
    assert a_uuid == from_dict(A, to_dict(a_uuid, reuse_instances=True), reuse_instances=True)

    a_int = A(1)
    assert a_int == from_dict(A, to_dict(a_int, reuse_instances=False), reuse_instances=False)
    assert a_int == from_dict(A, to_dict(a_int, reuse_instances=True), reuse_instances=True)

    a_str = A("hello")
    assert a_str == from_dict(A, to_dict(a_str, reuse_instances=False), reuse_instances=False)
    assert a_str == from_dict(A, to_dict(a_str, reuse_instances=True), reuse_instances=True)


def test_union_in_other_type() -> None:
    @serde
    class A:
        v: dict[str, Union[UUID, int]]

    a_uuid = A({"key": UUID("00611ee9-7ca3-41d3-9607-ea7268e264ea")})
    assert a_uuid == from_dict(A, to_dict(a_uuid, reuse_instances=False), reuse_instances=False)
    assert a_uuid == from_dict(A, to_dict(a_uuid, reuse_instances=True), reuse_instances=True)

    a_int = A({"key": 1})
    assert a_int == from_dict(A, to_dict(a_int, reuse_instances=False), reuse_instances=False)
    assert a_int == from_dict(A, to_dict(a_int, reuse_instances=True), reuse_instances=True)


def test_union_rename_all() -> None:
    @serde(rename_all="pascalcase")
    class Foo:
        bar_baz: Union[int, str]

    assert to_dict(Foo(10)) == {"BarBaz": 10}
    assert from_dict(Foo, {"BarBaz": "foo"}) == Foo("foo")


def test_union_with_list_of_other_class() -> None:
    @serde
    class A:
        a: int

    @serde
    class B:
        b: Union[list[A], str]

    b = B([A(1)])
    b_dict = {"b": [{"a": 1}]}
    assert to_dict(b) == b_dict
    assert from_dict(B, b_dict) == b


# relates to https://github.com/yukinarit/pyserde/issues/113
def test_union_with_union_in_nested_types() -> None:
    @serde
    class A:
        v: Union[UUID, list[Union[UUID, int]]]

    a_uuid = A([UUID("00611ee9-7ca3-41d3-9607-ea7268e264ea")])
    assert to_dict(a_uuid, reuse_instances=False) == {"v": ["00611ee9-7ca3-41d3-9607-ea7268e264ea"]}
    assert a_uuid == from_dict(A, to_dict(a_uuid, reuse_instances=False), reuse_instances=False)
    assert a_uuid == from_dict(A, to_dict(a_uuid, reuse_instances=True), reuse_instances=True)

    a_int = A([1])
    assert to_dict(a_int) == {"v": [1]}
    assert a_int == from_dict(A, to_dict(a_int, reuse_instances=False), reuse_instances=False)
    assert a_int == from_dict(A, to_dict(a_int, reuse_instances=True), reuse_instances=True)


# relates to https://github.com/yukinarit/pyserde/issues/113
def test_union_with_union_in_nested_tuple() -> None:
    @serde
    class A:
        v: Union[bool, tuple[Union[str, int]]]

    a_bool = A(False)
    a_bool_dict = {"v": False}
    assert to_dict(a_bool) == a_bool_dict
    assert from_dict(A, a_bool_dict) == a_bool

    a_str = A(("a",))
    a_str_dict = {"v": ("a",)}
    assert to_dict(a_str) == a_str_dict
    assert from_dict(A, a_str_dict) == a_str

    a_int = A((1,))
    a_int_dict = {"v": (1,)}
    assert to_dict(a_int) == a_int_dict
    assert from_dict(A, a_int_dict) == a_int


@pytest.mark.skip(reason="Known issue with generic union deserialization - TODO fix")
def test_generic_union() -> None:
    T = TypeVar("T")
    U = TypeVar("U")

    @serde
    class Foo(Generic[T]):
        v: T

    @serde
    class Bar(Generic[U]):
        v: U

    @serde
    class A:
        v: Union[Foo[int], Bar[str]]

    a = A(Foo[int](10))
    assert a == from_dict(A, to_dict(a))
    assert a == from_tuple(A, to_tuple(a))

    a = A(Bar[str]("foo"))
    assert a == from_dict(A, to_dict(a))
    assert a == from_tuple(A, to_tuple(a))

    @serde
    class B(Generic[T, U]):
        v: Union[Foo[T], Bar[U]]

    b = B[int, str](Foo[int](10))
    assert b == from_dict(B[int, str], to_dict(b))
    assert b == from_tuple(B[int, str], to_tuple(b))

    b = B[Foo[int], Bar[str]](Foo(Foo(10)))  # type: ignore[assignment]
    assert {"v": {"v": {"v": 10}}} == to_dict(b)
    # TODO Nested union generic still doesn't work
    # assert b == from_dict(B[Foo[int], Bar[str]], to_dict(b))


def test_external_tagging() -> None:
    @serde
    class Bar:
        b: int

    @serde
    class Baz:
        b: int

    @serde
    class Nested:
        v: Union[Bar, Baz]

    @serde
    class Foo:
        a: Union[Bar, Baz]
        b: Union[int, str]
        c: Union[dict[int, str], list[int]]
        d: Union[int, Nested]

    f = Foo(Bar(10), "foo", {10: "bar"}, Nested(Baz(100)))
    d = {
        "a": {
            "Bar": {"b": 10}
        },  # Union of dataclasses will be (de)serialized with external tagging
        "b": "foo",  # non dataclass will be untagged
        "c": {10: "bar"},
        "d": {"Nested": {"v": {"Baz": {"b": 100}}}},
    }
    assert to_dict(f) == d
    assert from_dict(Foo, d) == f

    @serde
    class FooMixed:
        a: Union[Bar, int]  # Mix of dataclass and non dataclass

    f_mixed = FooMixed(Bar(10))
    assert from_dict(FooMixed, to_dict(f_mixed)) == f_mixed
    f_mixed2 = FooMixed(10)
    assert from_dict(FooMixed, to_dict(f_mixed2)) == f_mixed2

    @serde
    class FooUnion:
        a: Union[Bar, Baz]

    f_union = FooUnion(Bar(10))
    assert f_union  # Use the variable

    # Tag not found
    with pytest.raises(SerdeError):
        assert from_dict(Foo, {"a": {"TagNotFound": {"b": 10}}})

    # Tag is correct, but incompatible data
    with pytest.raises(SerdeError):
        assert from_dict(Foo, {"a": {"Bar": {"c": 10}}})


def test_internal_tagging() -> None:
    from serde import InternalTagging

    @serde
    class Bar:
        v: int

    @serde
    class Baz:
        v: int

    @serde(tagging=InternalTagging("type"))
    class Nested:
        v: Union[Bar, Baz]

    @serde(tagging=InternalTagging("type"))
    class FooInternalComplete:
        a: Union[Bar, Baz]
        b: Union[int, str]
        c: Union[dict[int, str], list[int]]
        d: Union[int, Nested]

    f = FooInternalComplete(Bar(10), "foo", {10: "bar"}, Nested(Baz(100)))
    d = {
        "a": {
            "type": "Bar",
            "v": 10,
        },  # Union of dataclasses will be (de)serialized with internal tagging
        "b": "foo",  # non dataclass will be untagged
        "c": {10: "bar"},
        "d": {"type": "Nested", "v": {"type": "Baz", "v": 100}},
    }
    assert to_dict(f) == d
    assert from_dict(FooInternalComplete, d) == f

    @serde(tagging=InternalTagging("type"))
    class FooInternal:
        a: Union[Bar, Baz]

    f_internal = FooInternal(Bar(10))
    assert f_internal  # Use the variable

    # Tag not found
    with pytest.raises(SerdeError):
        assert from_dict(FooInternal, {"a": {"TagNotFound": "", "v": 10}})

    # Tag is correct, but incompatible data
    with pytest.raises(SerdeError):
        assert from_dict(FooInternal, {"a": {"type": "Bar", "c": 10}})

    with pytest.raises(TypeError):
        # Tag is not specified in attribute
        @serde(tagging=InternalTagging())  # type: ignore[call-overload]
        class FooErrorTest:
            pass


def test_adjacent_tagging() -> None:
    from serde import AdjacentTagging

    @serde
    class Bar:
        v: int

    @serde
    class Baz:
        v: int

    @serde(tagging=AdjacentTagging("type", "content"))
    class Nested:
        v: Union[Bar, Baz]

    @serde(tagging=AdjacentTagging("type", "content"))
    class Foo:
        a: Union[Bar, Baz]
        b: Union[int, str]
        c: Union[dict[int, str], list[int]]
        d: Union[int, Nested]

    f = Foo(Bar(10), "foo", {10: "bar"}, Nested(Baz(100)))
    d = {
        "a": {
            "type": "Bar",
            "content": {"v": 10},
        },  # Union of dataclasses will be (de)serialized with adjacent tagging
        "b": "foo",  # non dataclass will be untagged
        "c": {10: "bar"},
        "d": {"type": "Nested", "content": {"v": {"type": "Baz", "content": {"v": 100}}}},
    }
    assert to_dict(f) == d
    assert from_dict(Foo, d) == f

    @serde(tagging=AdjacentTagging("type", "content"))
    class FooAdjacent:
        a: Union[Bar, Baz]

    f_adjacent = FooAdjacent(Bar(10))
    assert f_adjacent  # Use the variable

    # Tag not found
    with pytest.raises(SerdeError):
        assert from_dict(FooAdjacent, {"a": {"TagNotFound": "", "content": {"v": 10}}})

    # Content tag not found
    with pytest.raises(SerdeError):
        assert from_dict(FooAdjacent, {"a": {"type": "Bar", "TagNotFound": {"v": 10}}})

    # Tag is correct, but incompatible data
    with pytest.raises(SerdeError):
        assert from_dict(FooAdjacent, {"a": {"type": "Bar", "content": {"c": 10}}})

    with pytest.raises(TypeError):
        # Tag is not specified in attribute
        @serde(tagging=AdjacentTagging(content="content"))  # type: ignore[call-overload]
        class FooContentError:
            pass

    with pytest.raises(TypeError):
        # Content is not specified in attribute
        @serde(tagging=AdjacentTagging(tag="tag"))  # type: ignore[call-overload]
        class FooTagError:
            pass

    with pytest.raises(TypeError):
        # Tag/Content is not specified in attribute
        @serde(tagging=AdjacentTagging())  # type: ignore[call-overload]
        class FooNoArgsError:
            pass


def test_untagged() -> None:
    from serde import Untagged

    @serde
    class Bar:
        v: int

    @serde
    class Baz:
        v: int

    @serde(tagging=Untagged)
    class Nested:
        v: Union[Bar, Baz]

    @serde(tagging=Untagged)
    class Foo:
        a: Union[Bar, Baz]
        b: Union[int, str]
        c: Union[dict[int, str], list[int]]
        d: Union[int, Nested]

    f = Foo(Bar(10), "foo", {10: "bar"}, Nested(Bar(100)))
    d = {"a": {"v": 10}, "b": "foo", "c": {10: "bar"}, "d": {"v": {"v": 100}}}
    assert to_dict(f) == d
    assert from_dict(Foo, d) == f

    @serde(tagging=Untagged)
    class FooUntagged:
        a: Union[Bar, Baz]

    f_untagged = FooUntagged(Baz(10))

    # Untagged can't differenciate the dataclass with similar fields
    with pytest.raises(AssertionError):
        assert to_dict(from_dict(FooUntagged, d)) == f_untagged  # type: ignore[comparison-overlap]

    # Untaggled can't differenciate dict and list.
    @serde
    class FooListDict:
        a: Union[list[int], dict[int, str]]

    f_listdict = FooListDict({10: "bar"})
    with pytest.raises(SerdeError):
        assert to_dict(from_dict(FooListDict, d)) == f_listdict  # type: ignore[comparison-overlap]


def test_newtype_and_untagged_union() -> None:
    """
    Regression test for
    https://github.com/yukinarit/pyserde/issues/292
    """
    from serde import Untagged

    OtherNewtypeThing = NewType("OtherNewtypeThing", str)

    @serde
    class SupposedlyUnrelated:
        name: OtherNewtypeThing

    @serde
    class Innerclass:
        inner_value: str = "SHOULD_NOT_APPEAR"

    @serde(tagging=Untagged)
    class MyDataclass:
        unrelateds: list[SupposedlyUnrelated]
        buggy_field: list[Union[str, Innerclass]]

    data = {"unrelateds": [], "buggy_field": [{"inner_value": "value"}, "something"]}
    actual = from_dict(MyDataclass, data)

    assert isinstance(actual.buggy_field[0], Innerclass)
    assert isinstance(actual.buggy_field[1], str)


def test_union_directly() -> None:
    @dataclass
    class Foo:
        v: int

    @dataclass
    class Bar:
        w: str

    bar = Bar("bar")

    # externally tagged
    s = to_json(bar, cls=Union[Foo, Bar])
    assert bar == from_json(Union[Foo, Bar], s)

    # internally tagged
    s = to_json(bar, cls=InternalTagging("type", Union[Foo, Bar]))
    assert bar == from_json(InternalTagging("type", Union[Foo, Bar]), s)

    # adjacently tagged
    s = to_json(bar, cls=AdjacentTagging("type", "content", Union[Foo, Bar]))
    assert bar == from_json(AdjacentTagging("type", "content", Union[Foo, Bar]), s)

    # untagged tagged
    s = to_json(bar, cls=Untagged(Union[Foo, Bar]))
    assert bar == from_json(Untagged(Union[Foo, Bar]), s)


def test_union_frozenset_with_prim() -> None:
    @serde
    @dataclass
    class Foo:
        a: Union[frozenset[int], int]

    assert to_dict(Foo(frozenset({1}))) == {"a": {1}}


def test_union_with_any() -> None:
    @dataclass
    class FooWithString:
        foo: str

    @dataclass
    class BarWithDict:
        bar: dict[str, Any]

    @serde(tagging=Untagged)
    @dataclass
    class Class:
        foobars: list[Union[FooWithString, BarWithDict]]

    c = Class([FooWithString("string"), BarWithDict({"key": "value"})])
    assert c == from_json(Class, to_json(c))


def test_union_with_different_ordering() -> None:

    @serde
    class Expr:
        expr: str

    @serde
    class Foo:
        a: Union[str, Expr]
        b: Union[Expr, str]
        c: Union[Expr, str]

    f = Foo("1", Expr("2"), "3")
    s = '{"a":"1","b":{"Expr":{"expr":"2"}},"c":"3"}'
    assert s == to_json(f)
    assert f == from_json(Foo, s)


def test_union_internal_tagging_for_non_dataclass() -> None:
    @serde
    class Bar:
        a: int

    @serde(tagging=InternalTagging("type"))
    class Foo:
        x: Union[list[int], Bar]

    f = Foo(Bar(1))
    assert f == from_json(Foo, to_json(f))

    f = Foo([10])
    assert f == from_json(Foo, to_json(f))


def test_union_internal_tagging_cache_conflict_different_case() -> None:
    @serde
    class Foo:
        v: int

    @serde
    class Bar:
        v: int

    union_upper = InternalTagging("TAG")(Union[Foo, Bar])
    deserialized_upper = Foo(1)
    serialized_upper = '{"v":1,"TAG":"Foo"}'
    assert from_json(union_upper, serialized_upper) == deserialized_upper
    assert to_json(deserialized_upper, union_upper) == serialized_upper

    union_lower = InternalTagging("tag")(Union[Foo, Bar])
    deserialized_lower = Foo(1)
    serialized_lower = '{"v":1,"tag":"Foo"}'
    assert from_json(union_lower, serialized_lower) == deserialized_lower
    assert to_json(deserialized_lower, union_lower) == serialized_lower



from serde.compat import SerdeError
from typing import Literal

from serde import serde
from serde.json import from_json, to_json


@serde
class Foo:
    i: Literal[1, 2]
    s: Literal["a", "b"]


def main() -> None:
    f = Foo(1, "a")
    print(f"Into Json: {to_json(f)}")

    s = '{"i": 2, "s": "b"}'
    print(f"From Json: {from_json(Foo, s)}")

    t = '{"i": 3, "s": "a"}'
    try:
        from_json(Foo, t)
    except SerdeError as err:
        print(f"Cannot parse '{t}' as Foo: {err}")

    u = '{"i": 2, "s": "A"}'
    try:
        from_json(Foo, u)
    except SerdeError as err:
        print(f"Cannot parse '{u}' as Foo: {err}")


if __name__ == "__main__":
    main()



import json
from typing import Any

SMALL = '{"i": 10, "s": "foo", "f": 100.0, "b": true}'

SMALL_TUPLE = (10, "foo", 100.0, True)

SMALL_DICT = json.loads(SMALL)

MEDIUM = f'{{"inner": [{",".join([SMALL] * 50)}]}}'

MEDIUM_TUPLE = ([[SMALL_TUPLE] * 50],)

json_md = (
    '{"i": 10, "s": "foo", "f": 100.0, "b": true,'
    '"i2": 10, "s2": "foo", "f2": 100.0, "b2": true,'
    '"i3": 10, "s3": "foo", "f3": 100.0, "b3": true,'
    '"i4": 10, "s4": "foo", "f4": 100.0, "b4": true,'
    '"i5": 10, "s5": "foo", "f5": 100.0, "b5": true}'
)

json_pri_container = (
    '{"v": [1, 2, 3, 4, 5], "d": {"foo": 10, "fuga": 20}, "foo": 30, "t": [true, false, true]}'
)

args_sm = {"i": 10, "s": "foo", "f": 100.0, "b": True}

args_md = [args_sm] * 50

# Large test data with lists, dicts, nested structures and unions
json_large_nested = (
    '{"customer_id": 12345, "name": "John Smith", "email": "john@example.com", '
    '"address": {"street": "123 Main St", "city": "New York", '
    '"country": "USA", "postal_code": "10001"}, '
    '"orders": [{"order_id": "ORD-001", "items": [{"product_id": 1, "name": "Laptop", '
    '"price": 999.99, "quantity": 1, "tags": ["electronics", "computer"]}, '
    '{"product_id": 2, "name": "Mouse", "price": 29.99, "quantity": 2, '
    '"tags": ["electronics", "accessory"]}], '
    '"total": 1059.97, "status": "shipped", '
    '"metadata": {"priority": "high", "notes": "Express delivery"}}, '
    '{"order_id": "ORD-002", "items": [{"product_id": 3, "name": "Book", '
    '"price": 19.99, "quantity": 3, "tags": ["books", "education"]}], '
    '"total": 59.97, "status": "processing", "metadata": {"priority": "normal", "notes": null}}], '
    '"preferences": {"theme": "dark", "notifications": true, '
    '"language": "en", "max_budget": 5000}, '
    '"loyalty_points": 1250, "created_at": "2024-01-15T10:30:00Z"}'
)

# Generate LARGE JSON data programmatically to avoid long lines


def _create_large_data() -> dict[str, Any]:
    items_list = ["laptop", "mouse", "keyboard", "monitor", "speakers"] * 20
    nested_data = {
        "category_1": list(range(50)),
        "category_2": list(range(50, 100)),
        "category_3": list(range(100, 150)),
        "category_4": list(range(150, 200)),
        "category_5": list(range(200, 250)),
    }

    return {
        "customer_id": 12345,
        "name": "John Smith",
        "email": "john@example.com",
        "preferences": {
            "theme": "dark",
            "notifications": True,
            "language": "en",
            "max_budget": 5000,
            "auto_renew": False,
            "privacy_level": 3,
        },
        "items_list": items_list,
        "nested_data": nested_data,
        "loyalty_points": 1250,
        "created_at": "2024-01-15T10:30:00Z",
    }


LARGE = json.dumps(_create_large_data())

# Complex nested args for large data
args_large = {
    "customer_id": 12345,
    "name": "John Smith",
    "email": "john@example.com",
    "address": {
        "street": "123 Main St",
        "city": "New York",
        "country": "USA",
        "postal_code": "10001",
    },
    "orders": [
        {
            "order_id": "ORD-001",
            "items": [
                {
                    "product_id": 1,
                    "name": "Laptop",
                    "price": 999.99,
                    "quantity": 1,
                    "tags": ["electronics", "computer"],
                },
                {
                    "product_id": 2,
                    "name": "Mouse",
                    "price": 29.99,
                    "quantity": 2,
                    "tags": ["electronics", "accessory"],
                },
            ],
            "total": 1059.97,
            "status": "shipped",
            "metadata": {"priority": "high", "notes": "Express delivery"},
        },
        {
            "order_id": "ORD-002",
            "items": [
                {
                    "product_id": 3,
                    "name": "Book",
                    "price": 19.99,
                    "quantity": 3,
                    "tags": ["books", "education"],
                }
            ],
            "total": 59.97,
            "status": "processing",
            "metadata": {"priority": "normal", "notes": None},
        },
    ],
    "preferences": {"theme": "dark", "notifications": True, "language": "en", "max_budget": 5000},
    "loyalty_points": 1250,
    "created_at": "2024-01-15T10:30:00Z",
}



import pytest
from decimal import Decimal
from typing import Union, Optional
from serde import serde, SerdeError, field
from serde.json import from_json
from serde.de import deserialize, from_obj, from_dict, Renderer, DeField


def test_from_obj() -> None:
    # Primitive
    assert 10 == from_obj(int, 10, False, False)

    # Union
    assert "a" == from_obj(Union[int, str], "a", False, False)  # type: ignore

    # tuple
    assert ("a", "b") == from_obj(tuple[str, str], ("a", "b"), False, False)

    # pyserde converts to the specified type
    assert 10 == from_obj(int, "10", False, False)

    # pyserde can converts for container types e.g. tuple, List etc.
    assert (1, 2) == from_obj(tuple[int, int], ("1", "2"), False, False)

    # Decimal
    dec = from_obj(list[Decimal], ("0.1", 0.1), False, False)
    assert isinstance(dec[0], Decimal) and dec[0] == Decimal("0.1")
    assert isinstance(dec[1], Decimal) and dec[1] == Decimal(0.1)


kwargs = (
    "maybe_generic=maybe_generic, maybe_generic_type_vars=maybe_generic_type_vars, "
    "variable_type_args=None, reuse_instances=reuse_instances, "
    "deserialize_numbers=deserialize_numbers"
)


def test_deserialize_numbers_option() -> None:
    def coerce_numbers(val: Union[str, int]) -> float:
        if isinstance(val, bool):
            raise SerdeError("bool not allowed")
        return float(val)

    @serde
    class Foo:
        value: float
        values: list[float]

    foo = from_dict(
        Foo,
        {"value": "1.0", "values": ["1", 2]},
        deserialize_numbers=coerce_numbers,
    )
    assert foo.value == 1.0
    assert foo.values == [1.0, 2.0]

    with pytest.raises(SerdeError):
        from_dict(
            Foo,
            {"value": True, "values": []},
            deserialize_numbers=coerce_numbers,
        )


def test_render_primitives() -> None:

    rendered = Renderer("foo").render(DeField(int, "i", datavar="data"))
    assert rendered == 'coerce_object("None", "i", int, data["i"])'

    rendered = Renderer("foo").render(DeField(int, "int_field", datavar="data", case="camelcase"))
    assert rendered == 'coerce_object("None", "int_field", int, data["intField"])'

    rendered = Renderer("foo").render(DeField(int, "i", datavar="data", index=1, iterbased=True))
    assert rendered == 'coerce_object("None", "i", int, data[1])'


def test_render_list() -> None:
    rendered = Renderer("foo").render(DeField(list[int], "l", datavar="data"))
    assert rendered == '[coerce_object("None", "v", int, v) for v in data["l"]]'

    rendered = Renderer("foo").render(DeField(list[list[int]], "l", datavar="data"))
    assert rendered == '[[coerce_object("None", "v", int, v) for v in v] for v in data["l"]]'


def test_render_tuple() -> None:
    @deserialize
    class Foo:
        pass

    rendered = Renderer("foo").render(DeField(tuple[str, int, list[int], Foo], "d", datavar="data"))
    rendered_str = 'coerce_object("None", "data[\\"d\\"][0]", str, data["d"][0])'
    rendered_int = 'coerce_object("None", "data[\\"d\\"][1]", int, data["d"][1])'
    rendered_lst = '[coerce_object("None", "v", int, v) for v in data["d"][2]]'
    rendered_foo = f"Foo.__serde__.funcs['foo'](data=data[\"d\"][3], {kwargs})"
    assert rendered == f"({rendered_str}, {rendered_int}, {rendered_lst}, {rendered_foo},)"

    field = DeField(tuple[str, int, list[int], Foo], "d", datavar="data", index=0, iterbased=True)
    rendered = Renderer("foo").render(field)
    rendered_str = 'coerce_object("None", "data[0][0]", str, data[0][0])'
    rendered_int = 'coerce_object("None", "data[0][1]", int, data[0][1])'
    rendered_lst = '[coerce_object("None", "v", int, v) for v in data[0][2]]'
    rendered_foo = f"Foo.__serde__.funcs['foo'](data=data[0][3], {kwargs})"
    assert rendered == f"({rendered_str}, {rendered_int}, {rendered_lst}, {rendered_foo},)"


def test_render_dict() -> None:
    rendered = Renderer("foo").render(DeField(dict[str, int], "d", datavar="data"))
    rendered_key = 'coerce_object("None", "k", str, k)'
    rendered_val = 'coerce_object("None", "v", int, v)'
    rendered_dct = f'{{{rendered_key}: {rendered_val} for k, v in data["d"].items()}}'
    assert rendered == rendered_dct

    @deserialize
    class Foo:
        pass

    rendered = Renderer("foo").render(DeField(dict[Foo, list[Foo]], "f", datavar="data"))
    rendered_key = f"Foo.__serde__.funcs['foo'](data=k, {kwargs})"
    rendered_val = f"[Foo.__serde__.funcs['foo'](data=v, {kwargs}) for v in v]"

    assert rendered == f'{{{rendered_key}: {rendered_val} for k, v in data["f"].items()}}'


def test_render_set() -> None:
    from typing import Set

    rendered = Renderer("foo").render(DeField(Set[int], "l", datavar="data"))
    assert rendered == 'set(coerce_object("None", "v", int, v) for v in data["l"])'

    rendered = Renderer("foo").render(DeField(Set[Set[int]], "l", datavar="data"))
    assert rendered == 'set(set(coerce_object("None", "v", int, v) for v in v) for v in data["l"])'


def test_render_opt() -> None:
    rendered = Renderer("foo").render(DeField(Optional[int], "o", datavar="data"))  # type: ignore
    rendered_opt = (
        '(coerce_object("None", "o", int, data["o"])) if data.get("o") is not None else None'
    )
    assert rendered == rendered_opt

    rendered = Renderer("foo").render(DeField(Optional[list[int]], "o", datavar="data"))  # type: ignore
    rendered_lst = '[coerce_object("None", "v", int, v) for v in data["o"]]'
    rendered_opt = f'({rendered_lst}) if data.get("o") is not None else None'
    assert rendered == rendered_opt

    rendered = Renderer("foo").render(DeField(Optional[list[int]], "o", datavar="data"))  # type: ignore
    rendered_lst = '[coerce_object("None", "v", int, v) for v in data["o"]]'
    rendered_opt = f'({rendered_lst}) if data.get("o") is not None else None'
    assert rendered == rendered_opt

    @deserialize
    class Foo:
        a: Optional[list[int]]

    rendered = Renderer("foo").render(DeField(Optional[Foo], "f", datavar="data"))  # type: ignore
    rendered_foo = f"Foo.__serde__.funcs['foo'](data=data[\"f\"], {kwargs})"
    rendered_opt = f'({rendered_foo}) if data.get("f") is not None else None'
    assert rendered == rendered_opt


def test_deny_unknown_fields() -> None:
    @serde(deny_unknown_fields=True)
    class Foo:
        a: int
        b: str

    with pytest.raises(SerdeError):
        from_json(Foo, '{"a": 10, "b": "foo", "c": 100.0, "d": true}')

    f = from_json(Foo, '{"a": 10, "b": "foo"}')
    assert f.a == 10
    assert f.b == "foo"


def test_deny_renamed_unknown_fields() -> None:
    @serde(deny_unknown_fields=True)
    class Foo:
        a: int
        b: str = field(rename="B")

    with pytest.raises(SerdeError):
        from_json(Foo, '{"a": 10, "b": "foo"}')

    f = from_json(Foo, '{"a": 10, "B": "foo"}')
    assert f.a == 10
    assert f.b == "foo"

    @serde(rename_all="constcase", deny_unknown_fields=True)
    class Bar:
        a: int
        b: str

    with pytest.raises(SerdeError):
        from_json(Bar, '{"a": 10, "b": "foo"}')

    b = from_json(Bar, '{"A": 10, "B": "foo"}')
    assert b.a == 10
    assert b.b == "foo"


def test_deny_aliased_unknown_fields() -> None:
    @serde(deny_unknown_fields=True)
    class Foo:
        a: int
        b: str = field(alias=["B"])  # type: ignore

    with pytest.raises(SerdeError):
        from_json(Foo, '{"a": 10, "b": "foo", "c": 100.0, "d": true}')

    f = from_json(Foo, '{"a": 10, "b": "foo"}')
    assert f.a == 10
    assert f.b == "foo"

    f = from_json(Foo, '{"a": 10, "B": "foo"}')
    assert f.a == 10
    assert f.b == "foo"



from datetime import datetime
from typing import Type, Any

from serde import SerdeSkip, default_deserializer, default_serializer, field, serde
from serde.json import from_json, to_json


def serializer(cls: Type[Any], o: Any) -> str:
    """
    Custom class level serializer.
    """

    # You can provide a custom serialize/deserialize logic for certain types.
    if cls is datetime:
        # I don't know why but this is untyped in typeshed
        return o.strftime("%d/%m/%y")  # type: ignore
    # Raise SerdeSkip to tell serde to use the default serializer/deserializer.
    else:
        raise SerdeSkip()


def deserializer(cls: Type[Any], o: Any) -> datetime:
    """
    Custom class level deserializer.
    """
    if cls is datetime:
        return datetime.strptime(o, "%d/%m/%y")
    else:
        raise SerdeSkip()


@serde(serializer=serializer, deserializer=deserializer)
class Foo:
    a: datetime
    # Override by field serializer/deserializer.
    b: datetime = field(
        serializer=lambda x: x.strftime("%y.%m.%d"),
        deserializer=lambda x: datetime.strptime(x, "%y.%m.%d"),
    )
    # Override by the default serializer/deserializer.
    c: datetime = field(serializer=default_serializer, deserializer=default_deserializer)


def main() -> None:
    dt = datetime(2021, 1, 1, 0, 0, 0)
    f = Foo(dt, dt, dt)
    print(f"Into Json: {to_json(f)}")

    s = '{"a": "01/01/21", "b": "01.01.21", "c": "2021-01-01T00:00:00"}'
    print(f"From Json: {from_json(Foo, s)}")


if __name__ == "__main__":
    main()



"""
Tests for flatten attribute.
"""

from typing import Any, Optional

import pytest

from serde import field, serde, SerdeError
from serde.json import from_json, to_json

from .common import all_formats


def test_flatten_simple() -> None:
    @serde
    class Bar:
        c: float
        d: bool

    @serde
    class Foo:
        a: int
        b: str
        bar: Bar = field(flatten=True)

    f = Foo(a=10, b="foo", bar=Bar(c=100.0, d=True))
    s = '{"a":10,"b":"foo","c":100.0,"d":true}'
    assert to_json(f) == s
    assert from_json(Foo, s) == f


@pytest.mark.parametrize("se,de", all_formats)
def test_flatten(se: Any, de: Any) -> None:
    @serde
    class Baz:
        e: list[int]
        f: dict[str, str]

    @serde
    class Bar:
        c: float
        d: bool
        baz: Baz = field(flatten=True)

    @serde
    class Foo:
        a: int
        b: str
        bar: Bar = field(flatten=True)

    f = Foo(a=10, b="foo", bar=Bar(c=100.0, d=True, baz=Baz([1, 2], {"a": "10"})))
    assert de(Foo, se(f)) == f


@pytest.mark.parametrize("se,de", all_formats)
def test_flatten_optional(se: Any, de: Any) -> None:
    @serde
    class Bar:
        c: float
        d: bool

    @serde
    class Foo:
        a: int
        b: str
        bar: Optional[Bar] = field(flatten=True)

    f = Foo(a=10, b="foo", bar=Bar(c=100.0, d=True))
    assert de(Foo, se(f)) == f


@pytest.mark.parametrize("se,de", all_formats)
def test_flatten_not_supported(se: Any, de: Any) -> None:
    @serde
    class Bar:
        pass

    with pytest.raises(SerdeError):

        @serde
        class Foo:
            bar: list[Bar] = field(flatten=True)


def test_flatten_default() -> None:
    @serde
    class Bar:
        c: float = field(default=0.0)
        d: bool = field(default=False)

    @serde
    class Foo:
        a: int
        b: str = field(default="foo")
        bar: Bar = field(flatten=True, default_factory=Bar)

    f = Foo(a=10, b="b", bar=Bar(c=100.0, d=True))
    assert from_json(Foo, to_json(f)) == f

    assert from_json(Foo, '{"a": 20}') == Foo(20, "foo", Bar())


def test_flatten_default_alias() -> None:
    @serde
    class Bar:
        a: float = field(default=0.0, alias=["aa"])  # type: ignore
        b: bool = field(default=False, alias=["bb"])  # type: ignore

    @serde
    class Foo:
        bar: Bar = field(flatten=True, default_factory=Bar)

    f = Foo(bar=Bar(100.0, True))
    assert from_json(Foo, to_json(f)) == f

    assert from_json(Foo, '{"aa": 20.0, "bb": false}') == Foo(Bar(20.0, False))



import json
from functools import partial
from typing import Union

import dacite
import data
from dataclasses_class import LARGE, MEDIUM, SMALL, Large, Medium, Small
from runner import Runner, Size


def de(cls: type, data: str) -> Union[Small, Medium, Large]:
    return dacite.from_dict(data_class=cls, data=json.loads(data))


def new(size: Size) -> Runner:
    name = "dacite"
    if size == Size.Small:
        unp = SMALL
        pac = data.SMALL
        cls = Small
    elif size == Size.Medium:
        unp = MEDIUM
        pac = data.MEDIUM
        cls = Medium
    elif size == Size.Large:
        unp = LARGE
        pac = data.LARGE
        cls = Large
    return Runner(name, unp, None, partial(de, cls, pac), None, None)



import uuid

from serde import serde
from serde.json import from_json, to_json


@serde
class Foo:
    v1: uuid.UUID
    v3: uuid.UUID
    v4: uuid.UUID
    v5: uuid.UUID


def main() -> None:
    foo = Foo(
        v1=uuid.uuid1(),
        v3=uuid.uuid3(uuid.NAMESPACE_DNS, "pyserde.org"),
        v4=uuid.uuid4(),
        v5=uuid.uuid5(uuid.NAMESPACE_DNS, "pyserde.org"),
    )
    print(f"Into Json: {to_json(foo)}")

    s = """{
        "v1": "a8098c1a-f86e-11da-bd1a-00112444be1e",
        "v3": "6fa459ea-ee8a-3ca4-894e-db77e160355e",
        "v4": "916b3609-0015-41be-be4c-53b74cab0d03",
        "v5": "886313e1-3b8a-5372-9b90-0c9aee199e5d"
    }"""
    print(f"From Json: {from_json(Foo, s)}")


if __name__ == "__main__":
    main()



import json
from functools import partial
from typing import Any, Union

import data
import marshmallow as ms
from dataclasses_class import LARGE, MEDIUM, SMALL, Large, Medium, Small
from runner import Runner, Size


class SmallSchema(ms.Schema):
    class Meta:
        render_module = json

    i = ms.fields.Int()
    s = ms.fields.Str()
    f = ms.fields.Float()
    b = ms.fields.Boolean()

    @ms.post_load
    def make_small(self, data: dict[str, Any], **kwargs: Any) -> Small:
        return Small(**data)


class MediumSchema(ms.Schema):
    class Meta:
        render_module = json

    inner = ms.fields.List(ms.fields.Nested(SmallSchema))

    @ms.post_load
    def make_medium(self, data: dict[str, Any], **kwargs: Any) -> Medium:
        return Medium(list(data["inner"]))


class LargeSchema(ms.Schema):
    class Meta:
        render_module = json

    customer_id = ms.fields.Int()
    name = ms.fields.Str()
    email = ms.fields.Str()
    preferences = ms.fields.Dict()
    items_list = ms.fields.List(ms.fields.Str())
    nested_data = ms.fields.Dict()
    loyalty_points = ms.fields.Int()
    created_at = ms.fields.Str()

    @ms.post_load
    def make_large(self, data: dict[str, Any], **kwargs: Any) -> Large:
        return Large(**data)


def new(size: Size) -> Runner:
    name = "marshmallow"
    if size == Size.Small:
        unp = SMALL
        pac = data.SMALL
        schema = SmallSchema()
    elif size == Size.Medium:
        unp = MEDIUM
        pac = data.MEDIUM
        schema = MediumSchema()  # type: ignore[assignment]
    elif size == Size.Large:
        unp = LARGE
        pac = data.LARGE
        schema = LargeSchema()  # type: ignore[assignment]
    return Runner(
        name,
        unp,
        partial(se, schema, unp),
        partial(de, schema, pac),
        None,
        partial(asdict, schema, unp),
    )


def se(schema: ms.Schema, obj: Union[Small, Medium, Large]) -> str:
    return schema.dumps(obj)  # type: ignore[no-any-return]


def de(schema: ms.Schema, data: str) -> Union[Small, Medium, Large]:
    return schema.loads(data)


def asdict(schema: ms.Schema, obj: Union[Small, Medium, Large]) -> dict[str, Any]:
    return schema.dump(obj)  # type: ignore[no-any-return]



import pytest
from beartype import beartype
from beartype.roar import BeartypeCallHintParamViolation
from dataclasses import dataclass
from serde import serde, SerdeError


def test_raises_serde_error_instead_of_beartype_error() -> None:
    @serde
    class Foo:
        v: int

    with pytest.raises(SerdeError):
        Foo("foo")  # type: ignore


def test_raises_beartype_error_if_beartype_decorated() -> None:
    # If a class already has beartype, pyserde cannot set custom validation error.
    @serde
    @beartype
    @dataclass
    class Foo:
        v: int

    with pytest.raises(BeartypeCallHintParamViolation):
        Foo("foo")  # type: ignore



from serde import asdict, astuple, serialize, to_dict, to_tuple
from serde.json import to_json
from serde.msgpack import to_msgpack
from serde.se import SeField, Renderer
from serde.core import TO_ITER

from . import data
from .data import (
    Bool,
    Float,
    Int,
    NestedInt,
    NestedPri,
    NestedPriDict,
    NestedPriList,
    NestedPriTuple,
    Pri,
    PriDict,
    PriList,
    PriOpt,
    PriTuple,
    Str,
)


def test_asdict() -> None:
    p = Pri(10, "foo", 100.0, True)
    assert {"b": True, "f": 100.0, "i": 10, "s": "foo"} == to_dict(p)
    assert {"b": True, "f": 100.0, "i": 10, "s": "foo"} == asdict(p)


def test_astuple() -> None:
    assert data.PRI_TUPLE == to_tuple(data.PRI)
    assert data.PRI_TUPLE == astuple(data.PRI)
    assert data.PRILIST == to_tuple(PriList(*data.PRILIST))
    assert data.PRILIST == astuple(PriList(*data.PRILIST))
    assert data.NESTED_PRILIST_TUPLE == to_tuple(NestedPriList(*data.NESTED_PRILIST))
    assert data.NESTED_PRILIST_TUPLE == astuple(NestedPriList(*data.NESTED_PRILIST))


def test_se_func_iter() -> None:
    # Primitives
    assert (10,) == to_tuple(Int(10))
    assert (10.0,) == to_tuple(Float(10.0))
    assert ("10",) == to_tuple(Str("10"))
    assert (False,) == to_tuple(Bool(False))

    assert (10, "10", 10.0, False) == to_tuple(Pri(10, "10", 10.0, False))
    assert ((10,),) == to_tuple(NestedInt(Int(10)))
    assert ((10,), ("10",), (10.0,), (True,)) == to_tuple(
        NestedPri(Int(10), Str("10"), Float(10.0), Bool(True))
    )

    # List
    assert ([10], ["10"], [10.0], [False]) == to_tuple(PriList([10], ["10"], [10.0], [False]))
    assert ([(10,)], [("10",)], [(10.0,)], [(False,)]) == to_tuple(
        NestedPriList([Int(10)], [Str("10")], [Float(10.0)], [Bool(False)])
    )

    # Dict
    assert ({"i": 10}, {"s": "10"}, {"f": 10.0}, {"b": False}) == to_tuple(
        PriDict({"i": 10}, {"s": "10"}, {"f": 10.0}, {"b": False})
    )
    assert ({"i": 10}, {"s": "10"}, {"f": 10.0}, {"b": False}) == to_tuple(
        PriDict({"i": 10}, {"s": "10"}, {"f": 10.0}, {"b": False})
    )
    assert ({("i",): (10,)}, {("i",): ("10",)}, {("i",): (10.0,)}, {("i",): (True,)}) == to_tuple(
        NestedPriDict(
            {Str("i"): Int(10)},  # pyright: ignore[reportUnhashable]
            {Str("i"): Str("10")},  # pyright: ignore[reportUnhashable]
            {Str("i"): Float(10.0)},  # pyright: ignore[reportUnhashable]
            {Str("i"): Bool(True)},  # pyright: ignore[reportUnhashable]
        )
    )

    # tuple
    exp = (
        (10, 10, 10),
        ("10", "10", "10", "10"),
        (10.0, 10.0, 10.0, 10.0, 10.0),
        (False, False, False, False, False, False),
    )
    act = to_tuple(
        PriTuple(
            (10, 10, 10),
            ("10", "10", "10", "10"),
            (10.0, 10.0, 10.0, 10.0, 10.0),
            (False, False, False, False, False, False),
        )
    )
    assert exp == act

    exp = (
        ((10,), (10,), (10,)),
        (("10",), ("10",), ("10",), ("10",)),
        ((10.0,), (10.0,), (10.0,), (10.0,), (10.0,)),
        ((False,), (False,), (False,), (False,), (False,), (False,)),
    )  # type: ignore
    act = to_tuple(
        NestedPriTuple(
            (Int(10), Int(10), Int(10)),
            (Str("10"), Str("10"), Str("10"), Str("10")),
            (Float(10.0), Float(10.0), Float(10.0), Float(10.0), Float(10.0)),
            (Bool(False), Bool(False), Bool(False), Bool(False), Bool(False), Bool(False)),
        )
    )
    assert exp == act

    # Optional
    assert (10, "10", 10.0, False) == to_tuple(PriOpt(10, "10", 10.0, False))


def test_convert_sets_option() -> None:
    @serialize
    class A:
        v: set[str]

    a = A({"a", "b"})

    a_json = to_json(a)
    # sets are unordered so the list order is not stable
    assert a_json == '{"v":["a","b"]}' or a_json == '{"v":["b","a"]}'

    a_msgpack = to_msgpack(a)
    # sets are unordered so the list order is not stable
    assert a_msgpack == b"\x81\xa1v\x92\xa1a\xa1b" or a_msgpack == b"\x81\xa1v\x92\xa1b\xa1a"

    a_dict = to_dict(a, convert_sets=True)
    # sets are unordered so the list order is not stable
    assert a_dict == {"v": ["a", "b"]} or a_dict == {"v": ["b", "a"]}

    assert {"v": {"a", "b"}} == to_dict(a, convert_sets=False)


@serialize
class Foo:
    val: int


kwargs = "reuse_instances=reuse_instances, convert_sets=convert_sets, skip_none=skip_none"


def test_render_primitives() -> None:
    rendered = Renderer(TO_ITER).render(SeField(int, "i"))
    assert rendered == 'coerce_object("None", "i", int, i)'


def test_render_list() -> None:

    rendered = Renderer(TO_ITER).render(SeField(list[int], "l"))
    assert rendered == '[coerce_object("None", "v", int, v) for v in l]'

    rendered = Renderer(TO_ITER).render(SeField(list[Foo], "foo"))
    assert rendered == f"[v.__serde__.funcs['to_iter'](v, {kwargs}) for v in foo]"


def test_render_dict() -> None:
    rendered = Renderer(TO_ITER).render(SeField(dict[str, Foo], "foo"))
    rendered_key = 'coerce_object("None", "k", str, k)'
    rendered_val = f"v.__serde__.funcs['to_iter'](v, {kwargs})"
    assert rendered == f"{{{rendered_key}: {rendered_val} for k, v in foo.items()}}"

    rendered = Renderer(TO_ITER).render(SeField(dict[Foo, Foo], "foo"))
    rendered_key = f"k.__serde__.funcs['to_iter'](k, {kwargs})"
    rendered_val = f"v.__serde__.funcs['to_iter'](v, {kwargs})"
    assert rendered == f"{{{rendered_key}: {rendered_val} for k, v in foo.items()}}"


def test_render_tuple() -> None:
    rendered = Renderer(TO_ITER).render(SeField(tuple[str, Foo, int], "foo"))
    rendered_str = 'coerce_object("None", "foo[0]", str, foo[0])'
    rendered_foo = f"foo[1].__serde__.funcs['to_iter'](foo[1], {kwargs})"
    rendered_int = 'coerce_object("None", "foo[2]", int, foo[2])'
    assert rendered == f"({rendered_str}, {rendered_foo}, {rendered_int},)"


def test_render_dataclass() -> None:
    rendered = Renderer(TO_ITER).render(SeField(Foo, "foo"))
    rendered_foo = f"foo.__serde__.funcs['to_iter'](foo, {kwargs})"
    assert rendered_foo == rendered



import pytest

import serde as serde_pkg
from serde import serde
from serde.yaml import to_yaml, from_yaml
from typing import Optional, Union


def test_yaml_basics() -> None:
    @serde
    class Foo:
        v: Optional[int]

    f = Foo(10)
    assert "v: 10\n" == to_yaml(f)
    assert f == from_yaml(Foo, "v: 10\n")

    @serde
    class Bar:
        v: set[int]

    b = Bar({1, 2, 3})
    assert "v:\n- 1\n- 2\n- 3\n" == to_yaml(b)
    assert b == from_yaml(Bar, "v:\n- 1\n- 2\n- 3\n")


def test_skip_none() -> None:
    @serde
    class Foo:
        a: int
        b: Optional[int]

    f = Foo(10, 100)
    assert (
        to_yaml(f, skip_none=True)
        == """\
a: 10
b: 100
"""
    )

    f = Foo(10, None)
    assert (
        to_yaml(f, skip_none=True)
        == """\
a: 10
"""
    )


def test_coerce_numbers_yaml() -> None:
    @serde
    class Foo:
        value: float

    foo = from_yaml(Foo, "value: 1\n")
    assert foo.value == 1.0

    @serde
    class Bar:
        values: list[float]

    bar = from_yaml(Bar, 'values:\n- 1\n- "2"\n')
    assert bar.values == [1.0, 2.0]

    baz = from_yaml(Foo, "value: 1e-3\n")
    assert baz.value == 0.001

    with pytest.raises(serde_pkg.SerdeError):
        from_yaml(Foo, "value: 1\n", coerce_numbers=False)


def test_yaml_numbers_with_union() -> None:
    @serde
    class Foo:
        value: Union[float, int]

    assert from_yaml(Foo, "value: 1\n").value == 1.0

    with pytest.raises(serde_pkg.SerdeError):
        from_yaml(Foo, 'value: "1"\n')

    @serde
    class Bar:
        value: Union[int, float]

    assert from_yaml(Bar, "value: 1\n").value == 1

    with pytest.raises(serde_pkg.SerdeError):
        from_yaml(Bar, 'value: "1"\n')



from serde import serde, SerdeError
from serde.json import from_json


@serde(deny_unknown_fields=True)
class Foo:
    a: int
    b: str


def main() -> None:
    try:
        s = '{"a": 10, "b": "foo", "c": 100.0, "d": true}'
        print(f"From Json: {from_json(Foo, s)}")
    except SerdeError:
        pass


if __name__ == "__main__":
    main()



"""
tomlfile.py

Read Pipenv Pipfile by pyserde.

Usage:
    $ uv sync
    $ uv run python tomlfile.py
"""

from pathlib import Path

from serde import Untagged, serde
from serde.toml import from_toml

basedir = Path(__file__).parent


@serde
class Source:
    url: str
    verify_ssl: bool
    name: str


@serde
class Requires:
    python_version: str


@serde
class Package:
    path: str | None = None
    version: str | None = None
    editable: bool | None = False


@serde(tagging=Untagged)
class Pipfile:
    source: list[Source]
    requires: Requires | None
    packages: dict[str, str | Package]


def main() -> None:
    with open(basedir / "Pipfile") as f:
        toml = f.read()
    pip = from_toml(Pipfile, toml)
    print(pip)


if __name__ == "__main__":
    main()



from __future__ import annotations
from serde import serde, to_dict, InternalTagging, from_dict
from dataclasses import dataclass


@serde(tagging=InternalTagging("type"))
@dataclass
class Leaf:
    value: int


@dataclass
class Node:
    name: str
    children: list[Leaf | Node]


serde(Node, tagging=InternalTagging("type"))


def main() -> None:
    node1 = Node("node1", [Leaf(10)])
    node2 = Node("node2", [node1])
    d = to_dict(node2)
    print(f"Into dict: {d}")
    node = from_dict(Node, d)
    print(f"From dict: {node}")


if __name__ == "__main__":
    main()



from plum import dispatch
from datetime import datetime
from serde import (
    serde,
    field,
)
from serde.json import from_json, to_json
from typing import Type, Any


class MySerializer:
    @dispatch
    def serialize(self, value: datetime) -> str:
        return value.strftime("%d/%m/%y")


class MyDeserializer:
    @dispatch
    def deserialize(self, cls: Type[datetime], value: Any) -> datetime:
        return datetime.strptime(value, "%d/%m/%y")


@serde(class_serializer=MySerializer(), class_deserializer=MyDeserializer())
class Foo:
    a: datetime
    b: datetime = field(
        serializer=lambda x: x.strftime("%y.%m.%d"),
        deserializer=lambda x: datetime.strptime(x, "%y.%m.%d"),
    )
    c: list[datetime] = field(default_factory=list)


def main() -> None:
    dt = datetime(2021, 1, 1, 0, 0, 0)
    f = Foo(dt, dt, [dt])
    print(f"Into Json: {to_json(f)}")

    s = '{"a": "01/01/21", "b": "01.01.21", "c": ["01/01/21"]}'
    print(f"From Json: {from_json(Foo, s)}")


if __name__ == "__main__":
    main()



from collections import defaultdict

from serde import serde
from serde.json import from_json, to_json


@serde
class Bar:
    v: int | None = None


@serde
class Foo:
    a: defaultdict[str, list[int]]
    b: defaultdict[str, int]
    c: defaultdict[str, Bar]


def main() -> None:
    a: defaultdict[str, list[int]] = defaultdict(list)
    a["a"].append(1)
    b: defaultdict[str, int] = defaultdict(int)
    b["b"]
    c: defaultdict[str, Bar] = defaultdict(Bar)
    c["c"].v = 10

    f = Foo(a, b, c)
    print(f"Into Json: {to_json(f)}")

    s = '{"a": {"a": [1, 2]}, "b": {"b": 10}, "c": {"c": {}}}'
    print(f"From Json: {from_json(Foo, s)}")


if __name__ == "__main__":
    main()



"""
rename_all.py


Usage:
    $ uv sync
    $ uv run python rename_all.py
"""

from serde import serde
from serde.json import from_json, to_json


@serde(rename_all="pascalcase")
class Foo:
    name: str
    no: int | None = None


def main() -> None:
    f = Foo("Pikachu")
    print(f"Into Json: {to_json(f)}")

    s = '{"Name": "Pikachu", "No": 25}'
    print(f"From Json: {from_json(Foo, s)}")


if __name__ == "__main__":
    main()



"""
Serialize and Deserialize in JSON format.
"""

from typing import Any, AnyStr, overload, Optional, Union, cast

from .compat import SerdeError, T
from .de import Deserializer, from_dict
from .se import Serializer, to_dict

# Lazy numpy imports to improve startup time

try:  # pragma: no cover
    import orjson

    def json_dumps(obj: Any, **opts: Any) -> str:
        if "option" in opts:
            opts["option"] |= orjson.OPT_NON_STR_KEYS
        else:
            opts["option"] = orjson.OPT_SERIALIZE_NUMPY | orjson.OPT_NON_STR_KEYS
        return cast(str, orjson.dumps(obj, **opts).decode())

    def json_loads(s: Union[str, bytes], **opts: Any) -> Any:
        return orjson.loads(s, **opts)

except ImportError:
    import json

    def json_dumps(obj: Any, **opts: Any) -> str:
        if "default" not in opts:
            from .numpy import encode_numpy

            opts["default"] = encode_numpy
        # compact output
        ensure_ascii = opts.pop("ensure_ascii", False)
        separators = opts.pop("separators", (",", ":"))
        return json.dumps(obj, ensure_ascii=ensure_ascii, separators=separators, **opts)

    def json_loads(s: Union[str, bytes], **opts: Any) -> Any:
        return json.loads(s, **opts)


__all__ = ["from_json", "to_json", "deserialize_json_numbers"]


class JsonSerializer(Serializer[str]):
    @classmethod
    def serialize(cls, obj: Any, **opts: Any) -> str:
        return json_dumps(obj, **opts)


class JsonDeserializer(Deserializer[AnyStr]):
    @classmethod
    def deserialize(cls, data: AnyStr, **opts: Any) -> Any:
        return json_loads(data, **opts)


def deserialize_json_numbers(value: Any) -> float:
    """
    Convert JSON numbers to float, accepting both ints and floats and rejecting non-numeric input.
    Useful when a JSON payload omits a decimal point but the target field is typed as float.
    """
    if isinstance(value, bool):
        raise SerdeError(f"Expected JSON number but got boolean {value!r}")
    if isinstance(value, (int, float)):
        return float(value)
    raise SerdeError(f"Expected JSON number but got {type(value).__name__}")


def to_json(
    obj: Any,
    cls: Optional[Any] = None,
    se: type[Serializer[str]] = JsonSerializer,
    reuse_instances: bool = False,
    convert_sets: bool = True,
    skip_none: bool = False,
    **opts: Any,
) -> str:
    """
    Serialize the object into JSON str. [orjson](https://github.com/ijl/orjson)
    will be used if installed.

    You can pass any serializable `obj`. If you supply other keyword arguments,
    they will be passed in `dumps` function.
    By default, numpy objects are serialized, this behaviour can be customized with the `option`
    argument with [orjson](https://github.com/ijl/orjson#numpy), or the `default` argument with
    Python standard json library.

    * `skip_none`: When set to True, any field in the class with a None value is excluded from the
    serialized output. Defaults to False.

    If you want to use another json package, you can subclass `JsonSerializer` and implement
    your own logic.
    """
    return se.serialize(
        to_dict(
            obj,
            c=cls,
            reuse_instances=reuse_instances,
            convert_sets=convert_sets,
            skip_none=skip_none,
        ),
        **opts,
    )


@overload
def from_json(
    c: type[T],
    s: AnyStr,
    de: type[Deserializer[AnyStr]] = JsonDeserializer,
    coerce_numbers: bool = True,
    **opts: Any,
) -> T: ...


# For Union, Optional etc.
@overload
def from_json(
    c: Any,
    s: AnyStr,
    de: type[Deserializer[AnyStr]] = JsonDeserializer,
    coerce_numbers: bool = True,
    **opts: Any,
) -> Any: ...


def from_json(
    c: Any,
    s: AnyStr,
    de: type[Deserializer[AnyStr]] = JsonDeserializer,
    coerce_numbers: bool = True,
    **opts: Any,
) -> Any:
    """
    Deserialize from JSON into the object. [orjson](https://github.com/ijl/orjson) will be used
    if installed.

    `c` is a class object and `s` is JSON bytes or str. If you supply other keyword arguments,
    they will be passed in `loads` function.

    * `coerce_numbers`: When True (default), ints from JSON are coerced to floats when the target
      type is float. Strings are never coerced.

    If you want to use another json package, you can subclass `JsonDeserializer` and implement your
    own logic.
    """
    deserialize_numbers = deserialize_json_numbers if coerce_numbers else None
    return from_dict(
        c,
        de.deserialize(s, **opts),
        reuse_instances=False,
        deserialize_numbers=deserialize_numbers,
    )



from serde import serde, field


@serde
class Foo:
    id: int = field(rename="ID")  # Field with attributes can be defined before other fields
    # thanks to dataclass_transform field_specifiers
    # https://peps.python.org/pep-0681/#the-dataclass-transform-decorator
    # NOTE: you still get error with mypy as of mypy v1.8.0
    comments: str


if __name__ == "__main__":
    pass



from serde import serde, from_dict, to_dict


type S = str


def test_pep695_type_alias() -> None:

    @serde
    class Foo:
        s: S

    f = Foo("foo")
    assert f == from_dict(Foo, to_dict(f))


@serde
class Bar:
    a: int


@serde
class Baz:
    b: int


type BarBaz = Bar | Baz


def test_pep695_type_alias_union() -> None:
    @serde
    class Foo:
        barbaz: BarBaz

    f = Foo(Baz(10))
    assert f == from_dict(Foo, to_dict(f))



import enum

import imported

from serde import serde
from serde.json import from_json, to_json


class Nested(enum.Enum):
    S = "foo"


class E(enum.Enum):
    S = "foo"
    F = 10.0
    B = True
    N = Nested.S


class IE(enum.IntEnum):
    V0 = enum.auto()
    V1 = enum.auto()
    V2 = enum.auto()


@serde
class Foo:
    v0: IE
    v1: IE = IE.V1  # Default enum value.
    v2: E = E.S
    v3: imported.ImportedEnum = imported.ImportedEnum.V3  # Use enum imported from other module.
    v4: E = E.N  # Use nested enum.


def main() -> None:
    f = Foo(IE.V0)
    s = to_json(f)
    print(s)

    f = from_json(Foo, s)
    print(f)
    s = to_json(f)

    s = to_json(Foo(IE(3)))
    print(s)


if __name__ == "__main__":
    main()



from serde import serde, field
from serde.json import from_json, to_json


@serde
class Foo:
    i: int = 10
    s: str = "foo"
    f: float = field(default=100.0)  # Use dataclass field.
    b: bool = field(default=True)
    d: dict[str, int] = field(default_factory=dict)


def main() -> None:
    h = Foo()
    print(f"Into Json: {to_json(h)}")

    s = "{}"
    print(f"From Json: {from_json(Foo, s)}")


if __name__ == "__main__":
    main()



import pathlib

from serde import serde
from serde.json import from_json, to_json


@serde
class Foo:
    p: pathlib.Path


def main() -> None:
    foo = Foo(pathlib.Path("foo/bar/baz.txt"))
    print(f"Into Json: {to_json(foo)}")

    s = '{"p": "foo/bar/baz.txt"}'
    print(f"From Json: {from_json(Foo, s)}")


if __name__ == "__main__":
    main()



import sys
import typing


if sys.version_info[:3] < (3, 12, 0):
    print("examples require at least Python 3.12")
    sys.exit(1)


def run_all() -> None:
    import alias
    import any
    import class_var
    import primitive_subclass
    import collection
    import custom_class_serializer
    import custom_legacy_class_serializer
    import custom_field_serializer
    import default
    import default_dict
    import env
    import flatten
    import forward_reference
    import frozen_set
    import generics
    import generics_pep695
    import generics_nested
    import nested
    import init_var
    import jsonfile
    import lazy_type_evaluation
    import literal
    import msg_pack
    import newtype
    import pep681
    import plain_dataclass
    import plain_dataclass_class_attribute
    import deny_unknown_fields
    import python_pickle
    import recursive
    import recursive_list
    import recursive_union
    import rename
    import rename_all
    import simple
    import skip
    import tomlfile
    import type_check_coerce
    import type_check_disabled
    import type_datetime
    import type_decimal
    import type_ipaddress
    import type_numpy
    import type_pathlib
    import type_uuid
    import type_alias_pep695
    import union
    import union_tagging
    import union_directly
    import user_exception
    import variable_length_tuple
    import yamlfile
    import enum34
    import kw_only

    run(any)
    run(simple)
    run(enum34)
    run(frozen_set)
    run(newtype)
    run(collection)
    run(default)
    run(default_dict)
    run(env)
    run(flatten)
    run(jsonfile)
    run(rename)
    run(rename_all)
    run(skip)
    run(tomlfile)
    run(yamlfile)
    run(union)
    run(custom_class_serializer)
    run(custom_legacy_class_serializer)
    run(custom_field_serializer)
    run(forward_reference)
    run(type_decimal)
    run(type_datetime)
    run(union_tagging)
    run(union_directly)
    run(generics)
    run(type_alias_pep695)
    run(generics_pep695)
    run(generics_nested)
    run(nested)
    run(lazy_type_evaluation)
    run(literal)
    run(type_check_coerce)
    run(type_check_disabled)
    run(user_exception)
    run(pep681)
    run(variable_length_tuple)
    run(init_var)
    run(python_pickle)
    run(class_var)
    run(alias)
    run(recursive)
    run(recursive_list)
    run(recursive_union)
    run(class_var)
    run(plain_dataclass)
    run(plain_dataclass_class_attribute)
    run(deny_unknown_fields)
    run(msg_pack)
    run(primitive_subclass)
    run(kw_only)
    run(type_pathlib)
    run(type_ipaddress)
    run(type_uuid)
    run(type_numpy)

    try:
        import type_sqlalchemy

        run(type_sqlalchemy)
    except ImportError:
        pass


def run(module: typing.Any) -> None:
    print("-----------------")
    print(f"running {module.__name__}")
    module.main()


if __name__ == "__main__":
    try:
        run_all()
        print("-----------------")
        print("all examples completed successfully!")
    except Exception:
        sys.exit(1)



import sys
from dataclasses import dataclass, field
from typing import Optional

import pytest

from serde import deserialize, from_dict


@pytest.mark.skipif(
    sys.version_info < (3, 10), reason="dataclasses `kw_only` requires python3.10 or higher"
)
def test_simple() -> None:
    @deserialize
    @dataclass(kw_only=True)
    class Hello:
        a: str

    assert Hello(a="ok") == from_dict(Hello, {"a": "ok"})


@pytest.mark.skipif(
    sys.version_info < (3, 10), reason="dataclasses `kw_only` requires python3.10 or higher"
)
def test_inheritance() -> None:
    @dataclass(kw_only=True)
    class Friend:
        name: str = "MyFriend"

    @dataclass(kw_only=True)
    class Parent:
        child_val: Optional[str]

    @dataclass(kw_only=True)
    class Child(Parent):
        value: int = 42
        friend: Friend = field(default_factory=Friend)

    # check with defaults
    assert Child(child_val="test") == from_dict(Child, {"child_val": "test"})

    # check without defaults
    assert Child(child_val="test", value=34, friend=Friend(name="my_friend")) == from_dict(
        Child, {"child_val": "test", "value": 34, "friend": {"name": "my_friend"}}
    )



import datetime
import decimal
import ipaddress
import itertools
import os
import pathlib
import uuid
from typing import (
    Any,
    Generic,
    NewType,
    Optional,
    TypeVar,
    Union,
    Callable,
)
from typing_extensions import TypeAlias

import more_itertools

from serde import from_dict, from_tuple, serde, to_dict, to_tuple
from serde.json import from_json, to_json
from serde.msgpack import from_msgpack, to_msgpack
from serde.pickle import from_pickle, to_pickle
from serde.toml import from_toml, to_toml
from serde.yaml import from_yaml, to_yaml
from tests import data

FormatFuncs: TypeAlias = list[tuple[Callable[..., Any], Callable[..., Any]]]

format_dict: FormatFuncs = [(to_dict, from_dict)]

format_tuple: FormatFuncs = [(to_tuple, from_tuple)]

format_json: FormatFuncs = [(to_json, from_json)]

format_msgpack: FormatFuncs = [(to_msgpack, from_msgpack)]

format_yaml: FormatFuncs = [(to_yaml, from_yaml)]

format_toml: FormatFuncs = [(to_toml, from_toml)]

format_pickle: FormatFuncs = [(to_pickle, from_pickle)]

all_formats: FormatFuncs = (
    format_dict
    + format_tuple
    + format_json
    + format_msgpack
    + format_yaml
    + format_toml
    + format_pickle
)

T = TypeVar("T")

U = TypeVar("U")

V = TypeVar("V")


@serde
class GenericClass(Generic[T, U]):
    a: T
    b: U


@serde
class Inner(Generic[T]):
    c: T


@serde
class NestedGenericClass(Generic[U, V]):
    a: U
    b: Inner[V]


Filter: TypeAlias = Callable[..., bool]


def param(val: T, typ: U, filter: Optional[Filter] = None) -> tuple[T, U, Filter]:
    """
    Create a test parameter

    * `val` is the expected value
    * `typ` is the expected type
    * If `filter` evaluates to True, it will filter this test case.
    """
    return (val, typ, filter or (lambda se, de, opt: False))


def toml_not_supported(se: Any, de: Any, opt: Any) -> bool:
    return se is to_toml


def yaml_not_supported(se: Any, de: Any, opt: Any) -> bool:
    return se is to_yaml


types: list[tuple[Any, Any, Any]] = [
    param(10, int),  # Primitive
    param("foo", str),
    param(100.0, float),
    param(True, bool),
    param(10, Optional[int]),  # Optional
    param(None, Optional[int], toml_not_supported),
    param([1, 2], list[int]),  # Container
    param([1, 2], list),
    param([], list[int]),
    param({1, 2}, set[int], toml_not_supported),
    param({1, 2}, set, toml_not_supported),
    param(set(), set[int], toml_not_supported),
    param(frozenset({1, 2}), frozenset[int], toml_not_supported),
    param((1, 1), tuple[int, int]),
    param((1, 1), tuple),
    param((1, 2, 3), tuple[int, ...]),
    param({"a": 1}, dict[str, int]),
    param({"a": 1}, dict),
    param({"a": 1}, dict),
    param({}, dict[str, int]),
    param({"a": 1}, dict[str, int]),
    # param({"a": 1}, defaultdict[str, int]),
    # param({"a": [1]}, defaultdict[str, list[int]]),
    param(data.Pri(10, "foo", 100.0, True), data.Pri),  # dataclass
    param(data.Pri(10, "foo", 100.0, True), Optional[data.Pri]),
    param(
        data.PrimitiveSubclass(data.StrSubclass("a")), data.PrimitiveSubclass, yaml_not_supported
    ),
    param(None, Optional[data.Pri], toml_not_supported),
    param(data.Recur(data.Recur(None, None, None), None, None), data.Recur, toml_not_supported),
    param(
        data.RecurContainer([data.RecurContainer([], {})], {"c": data.RecurContainer([], {})}),
        data.RecurContainer,
        toml_not_supported,
    ),
    param(data.Init(1), data.Init),
    param(10, NewType("Int", int)),  # NewType
    param({"a": 1}, Any),  # Any
    param(GenericClass[str, int]("foo", 10), GenericClass[str, int]),
    param(NestedGenericClass[str, int]("foo", Inner[int](10)), NestedGenericClass[str, int]),
    param(pathlib.Path("/tmp/foo"), pathlib.Path),  # Extended types
    param(pathlib.Path("/tmp/foo"), Optional[pathlib.Path]),
    param(None, Optional[pathlib.Path], toml_not_supported),
    param(pathlib.PurePath("/tmp/foo"), pathlib.PurePath),
    param(pathlib.PurePosixPath("/tmp/foo"), pathlib.PurePosixPath),
    param(pathlib.PureWindowsPath("C:\\tmp"), pathlib.PureWindowsPath),
    param(uuid.UUID("8f85b32c-a0be-466c-87eb-b7bbf7a01683"), uuid.UUID),
    param(ipaddress.IPv4Address("127.0.0.1"), ipaddress.IPv4Address),
    param(ipaddress.IPv6Address("::1"), ipaddress.IPv6Address),
    param(ipaddress.IPv4Network("127.0.0.0/8"), ipaddress.IPv4Network),
    param(ipaddress.IPv6Network("::/128"), ipaddress.IPv6Network),
    param(ipaddress.IPv4Interface("192.168.1.1/24"), ipaddress.IPv4Interface),
    param(ipaddress.IPv6Interface("::1/128"), ipaddress.IPv6Interface),
    param(decimal.Decimal(10), decimal.Decimal),
    param(datetime.datetime.strptime("Jan 1 2021 1:55PM", "%b %d %Y %I:%M%p"), datetime.datetime),
    param(
        datetime.datetime.strptime("Jan 1 2021 1:55PM", "%b %d %Y %I:%M%p").date(), datetime.date
    ),
    param(
        datetime.datetime.strptime("Jan 1 2021 1:55PM", "%b %d %Y %I:%M%p").time(), datetime.time
    ),
]

# these types can only be instantiated on their corresponding system
if os.name == "posix":
    types.append(param(pathlib.PosixPath("/tmp/foo"), pathlib.PosixPath))
if os.name == "nt":
    types.append(param(pathlib.WindowsPath("C:\\tmp"), pathlib.WindowsPath))

types_combinations: list[Any] = [
    list(more_itertools.flatten(c)) for c in itertools.combinations(types, 2)
]

opt_case: list[dict[str, Union[bool, str]]] = [
    {"reuse_instances_default": False},
    {"reuse_instances_default": False, "rename_all": "camelcase"},
    {"reuse_instances_default": False, "rename_all": "snakecase"},
]


def make_id_from_dict(d: dict[str, Union[bool, str]]) -> str:
    if not d:
        return "none"
    else:
        key = list(d)[0]
        return f"{key}-{d[key]}"


def opt_case_ids() -> list[str]:
    """
    Create parametrize test id
    """
    return list(map(make_id_from_dict, opt_case))


def type_ids() -> list[str]:
    """
    Create parametrize test id
    """
    from serde.compat import typename

    def make_id(pair: tuple[Any, ...]) -> str:
        t, T, _ = pair
        return f"{typename(T)}({t})"

    return list(map(make_id, types))


def type_combinations_ids() -> list[str]:
    """
    Create parametrize test id
    """
    from serde.compat import typename

    def make_id(quad: tuple[Any, ...]) -> str:
        t, T, u, U = quad
        return f"{typename(T)}({t})-{typename(U)}({u})"

    return list(map(make_id, types_combinations))



from serde import serde
from serde.json import from_json, to_json


@serde
class Bar:
    a: int


@serde
class Baz:
    b: int


# BarBaz = Bar | Baz
type BarBaz = Bar | Baz


@serde
class Foo:
    barbaz: BarBaz


def main() -> None:
    f = Foo(Baz(10))
    s = to_json(f)
    print(f"Into Json: {s}")
    ff = from_json(Foo, s)
    print(f"From Json: {ff}")


if __name__ == "__main__":
    main()



from serde import serde
from serde.json import from_json, to_json


@serde
class Foo:
    a: list[str]
    b: tuple[str, bool]
    c: dict[str, list[int]]


def main() -> None:
    h = Foo(["1", "2"], ("foo", True), {"bar": [10, 20]})
    print(f"Into Json: {to_json(h)}")

    s = '{"a": ["1", "2"], "b": ["foo", true], "c": {"bar": [10, 20]}}'
    print(f"From Json: {from_json(Foo, s)}")


if __name__ == "__main__":
    main()



from serde import serde
from serde.pickle import from_pickle, to_pickle


@serde
class Foo:
    i: int
    s: str
    f: float
    b: bool


def main() -> None:
    f = Foo(i=10, s="foo", f=100.0, b=True)
    bin = to_pickle(f)
    print("Into Pickle: ", bin)
    print(f"From Pickle: {from_pickle(Foo, bin)}")


if __name__ == "__main__":
    main()



from dataclasses import dataclass

from serde import serde
from serde.json import from_json, to_json


@serde
@dataclass(kw_only=True)
class Foo:
    i: int
    s: str
    f: float
    b: bool


def main() -> None:
    f = Foo(i=10, s="foo", f=100.0, b=True)
    print(f"Into Json: {to_json(f)}")

    s = '{"i": 10, "s": "foo", "f": 100.0, "b": true}'
    print(f"From Json: {from_json(Foo, s)}")


if __name__ == "__main__":
    main()



from typing import Generic, TypeVar

from serde import from_dict, serde, to_dict


@serde
class EventData:
    pass


@serde
class A(EventData):
    a: str


# Additional subclasses of EventData exist

Data = TypeVar("Data", bound=EventData)


@serde
class Payload(Generic[Data]):
    id: int
    data: Data


@serde
class Event(Generic[Data]):
    name: str
    payload: Payload[Data]


def main() -> None:
    event_a = Event("a", Payload(1, A("a_str")))

    a_dict = to_dict(event_a)
    new_event_a = from_dict(Event[A], a_dict)

    print(event_a)
    print(new_event_a)

    # This has a bug, see https://github.com/yukinarit/pyserde/issues/464
    # payload = Payload(1, A("a_str"))
    # payload_dict = to_dict(payload)
    # new_payload = from_dict(Payload[A], payload_dict)
    # print(payload)
    # print(new_payload)


if __name__ == "__main__":
    main()



from serde import serde
from serde.json import from_json, to_json


@serde
class Foo:
    v: tuple[int, ...]


def main() -> None:
    f = Foo(v=(1, 2, 3))
    print(f"Into Json: {to_json(f)}")

    s = '{"v": [1, 2, 3]}'
    print(f"From Json: {from_json(Foo, s)}")


if __name__ == "__main__":
    main()



from typing import Generic, TypeVar

from serde import from_dict, serde, to_dict

T = TypeVar("T")


@serde
class Bar:
    n: int


@serde
class Foo(Generic[T]):
    inner: T


@serde
class Baz(Generic[T]):
    foo: Foo[T]


def main() -> None:
    # Use dataclass as type parameter
    foobar = Foo[Bar](Bar(10))
    d = to_dict(foobar)
    print(from_dict(Foo[Bar], d))

    # Use primitive as type parameter
    fooint = Foo[int](10)
    d = to_dict(fooint)
    print(from_dict(Foo[int], d))

    # No type parameter. Any is deduced.
    foobar = Foo(Bar(10))
    d = to_dict(foobar)
    print(from_dict(Foo, d))

    # Nested
    bazint = Baz[int](Foo[int](10))
    d = to_dict(bazint)
    print(from_dict(Baz[int], d))


if __name__ == "__main__":
    main()



"""
rename.py


Usage:
    $ uv sync
    $ uv run python rename.py
"""

from serde import field, serde
from serde.json import to_json


@serde
class Foo:
    # Use 'class_name' because 'class' is a keyword.
    class_name: str = field(rename="class")


def main() -> None:
    print(to_json(Foo(class_name="Foo")))


if __name__ == "__main__":
    main()



from dataclasses import dataclass

from serde import serde
from serde.json import from_json, to_json


# Suppose this is a dataclass defined in an external library
# To (de)serialize this dataclass with a class attribute,
# create a new class `Wrapper` to extend `External` then add the
# class attribute to `Wrapper`.
@dataclass
class External:
    some_value: int


@serde(rename_all="kebabcase")
@dataclass
class Wrapper(External):
    pass


def main() -> None:
    f = Wrapper(some_value=10)
    print(f"Into Json: {to_json(f)}")

    s = '{"some-value": 10}'
    print(f"From Json: {from_json(Wrapper, s)}")


if __name__ == "__main__":
    main()



import ipaddress

from serde import serde
from serde.json import from_json, to_json


@serde
class Foo:
    v4: ipaddress.IPv4Address
    v6: ipaddress.IPv6Address
    v4n: ipaddress.IPv4Network
    v6n: ipaddress.IPv6Network


def main() -> None:
    foo = Foo(
        v4=ipaddress.IPv4Address("192.168.0.1"),
        v6=ipaddress.IPv6Address("2001:db8::"),
        v4n=ipaddress.IPv4Network("192.168.0.0/28"),
        v6n=ipaddress.IPv6Network("2001:db8::/32"),
    )
    print(f"Into Json: {to_json(foo)}")

    s = '{"v4": "192.168.0.1", "v6": "2001:db8::", "v4n": "192.168.0.0/28", "v6n": "2001:db8::/32"}'
    print(f"From Json: {from_json(Foo, s)}")


if __name__ == "__main__":
    main()



from typing import ClassVar

from serde import serde
from serde.json import from_json, to_json


@serde
class Bar:
    v: int


@serde(serialize_class_var=True)
class Foo:
    a: ClassVar[int] = 10
    b: ClassVar[Bar] = Bar(100)
    c: ClassVar[list[Bar]] = [Bar(1), Bar(2)]


def main() -> None:
    f = Foo()
    print(f"Into Json: {to_json(f)}")
    print(f"From Json: {from_json(Foo, '{}')}")


if __name__ == "__main__":
    main()



from serde import field, serde
from serde.json import from_json


@serde
class Foo:
    a: int = field(alias=["b", "c", "d"])


def main() -> None:
    s = '{"a": 10}'
    print(f"From Json: {from_json(Foo, s)}")

    s = '{"b": 20}'
    print(f"From Json: {from_json(Foo, s)}")

    s = '{"c": 30}'
    print(f"From Json: {from_json(Foo, s)}")

    s = '{"d": 40}'
    print(f"From Json: {from_json(Foo, s)}")

    try:
        s = '{"e": 50}'
        print(f"From Json: {from_json(Foo, s)}")
    except Exception:
        pass


if __name__ == "__main__":
    main()



# see https://docs.pytest.org/en/latest/example/pythoncollection.html#customizing-test-collection
import sys

collect_ignore = ["setup.py"]

if sys.version_info[:2] < (3, 12):
    collect_ignore.append("tests/test_type_alias.py")



"""
Tests for custom serializer/deserializer.
"""

from datetime import datetime
from typing import Optional, Any, Union
from plum import dispatch

import pytest

from serde import (
    SerdeError,
    field,
    from_tuple,
    serde,
    to_tuple,
    ClassSerializer,
    ClassDeserializer,
)
from serde.json import from_json, to_json


def test_custom_field_serializer() -> None:
    @serde
    class Foo:
        a: datetime
        b: datetime = field(
            serializer=lambda x: x.strftime("%d/%m/%y"),
            deserializer=lambda x: datetime.strptime(x, "%d/%m/%y"),
        )
        c: Optional[datetime] = field(
            serializer=lambda x: x.strftime("%d/%m/%y") if x else None,
            deserializer=lambda x: datetime.strptime(x, "%d/%m/%y") if x else None,
        )

    dt = datetime(2021, 1, 1, 0, 0, 0)
    f = Foo(dt, dt, None)

    assert to_json(f) == '{"a":"2021-01-01T00:00:00","b":"01/01/21","c":null}'
    assert f == from_json(Foo, to_json(f))

    assert to_tuple(f) == (datetime(2021, 1, 1, 0, 0), "01/01/21", None)
    assert f == from_tuple(Foo, to_tuple(f))


def test_raise_error() -> None:
    def raise_exception(_: Any) -> None:
        raise Exception()

    @serde
    class Foo:
        i: int = field(serializer=raise_exception, deserializer=raise_exception)

    f = Foo(10)
    with pytest.raises(SerdeError):
        to_json(f)

    with pytest.raises(SerdeError):
        from_json(Foo, '{"i": 10}')


def test_wrong_signature() -> None:
    @serde
    class Foo:
        i: int = field(serializer=lambda: "10", deserializer=lambda: 10)

    f = Foo(10)
    with pytest.raises(SerdeError):
        to_json(f)

    with pytest.raises(SerdeError):
        from_json(Foo, '{"i": 10}')


def test_custom_class_serializer() -> None:
    class MySerializer(ClassSerializer):
        @dispatch
        def serialize(self, value: datetime) -> str:
            return value.strftime("%d/%m/%y")

    class MyDeserializer(ClassDeserializer):
        @dispatch
        def deserialize(self, cls: type[datetime], value: Any) -> datetime:
            return datetime.strptime(value, "%d/%m/%y")

    @serde(class_serializer=MySerializer(), class_deserializer=MyDeserializer())
    class Bar:
        e: datetime
        f: list[datetime]

    @serde(class_serializer=MySerializer(), class_deserializer=MyDeserializer())
    class Foo:
        a: datetime
        b: list[datetime]
        c: dict[str, datetime]
        d: Bar

    dt = datetime(2021, 1, 1, 0, 0, 0)
    f = Foo(dt, [dt], {"k": dt}, Bar(dt, [dt]))

    s = (
        '{"a":"01/01/21","b":["01/01/21"],"c":{"k":"01/01/21"},'
        '"d":{"e":"01/01/21","f":["01/01/21"]}}'
    )
    assert s == to_json(f)
    assert f == from_json(Foo, s)

    t = ("01/01/21", ["01/01/21"], {"k": "01/01/21"}, ("01/01/21", ["01/01/21"]))
    assert t == to_tuple(f)
    assert f == from_tuple(Foo, t)


def test_custom_class_serializer_dataclass() -> None:
    @serde
    class Bar:
        v: int

    class MySerializer(ClassSerializer):
        @dispatch
        def serialize(self, value: Bar) -> str:
            return str(value.v)

    class MyDeserializer(ClassDeserializer):
        @dispatch
        def deserialize(self, cls: type[Bar], value: Any) -> Bar:
            return Bar(int(value))

    @serde(class_serializer=MySerializer(), class_deserializer=MyDeserializer())
    class Foo:
        bar: Bar

    f = Foo(Bar(10))

    s = '{"bar":"10"}'
    assert s == to_json(f)
    assert f == from_json(Foo, s)


def test_custom_serializer_with_field_attributes() -> None:
    class MySerializer(ClassSerializer):
        @dispatch
        def serialize(self, value: datetime) -> str:
            return value.strftime("%d/%m/%y")

    class MyDeserializer(ClassDeserializer):
        @dispatch
        def deserialize(self, cls: type[datetime], value: Any) -> datetime:
            return datetime.strptime(value, "%d/%m/%y")

    @serde(class_serializer=MySerializer(), class_deserializer=MyDeserializer())
    class Foo:
        # TODO This does not work as expected
        # a: Optional[datetime] = field(
        #    serializer=lambda x: x.strftime("%d/%m/%y"),
        #    deserializer=lambda x: datetime.strptime(x, "%d/%m/%y"),
        #    skip=True,
        # )
        b: Optional[datetime] = field(skip_if_false=True)
        c: Optional[datetime] = field(skip_if_false=True)

    dt = datetime(2021, 1, 1, 0, 0, 0)
    f = Foo(dt, None)
    s = '{"b":"01/01/21"}'
    assert s == to_json(f)
    assert f == from_json(Foo, s)


def test_custom_serializer_with_class_attributes() -> None:
    class MySerializer(ClassSerializer):
        @dispatch
        def serialize(self, value: datetime) -> str:
            return value.strftime("%d/%m/%y")

    class MyDeserializer(ClassDeserializer):
        @dispatch
        def deserialize(self, cls: type[datetime], value: Any) -> datetime:
            return datetime.strptime(value, "%d/%m/%y")

    @serde(
        class_serializer=MySerializer(),
        class_deserializer=MyDeserializer(),
        rename_all="capitalcase",
    )
    class Foo:
        a: datetime
        b: datetime = field(
            serializer=lambda x: x.strftime("%d/%m/%y"),
            deserializer=lambda x: datetime.strptime(x, "%d/%m/%y"),
        )

    dt = datetime(2021, 1, 1, 0, 0, 0)
    f = Foo(dt, dt)
    s = '{"A":"01/01/21","B":"01/01/21"}'
    assert s == to_json(f)
    assert f == from_json(Foo, s)


def test_field_serialize_override_class_serializer() -> None:
    class MySerializer(ClassSerializer):
        @dispatch
        def serialize(self, value: datetime) -> str:
            return value.strftime("%d/%m/%y")

    class MyDeserializer(ClassDeserializer):
        @dispatch
        def deserialize(self, cls: type[datetime], value: Any) -> datetime:
            return datetime.strptime(value, "%d/%m/%y")

    @serde(class_serializer=MySerializer(), class_deserializer=MyDeserializer())
    class Foo:
        a: int
        b: datetime
        c: datetime = field(
            serializer=lambda x: x.strftime("%y.%m.%d"),
            deserializer=lambda x: datetime.strptime(x, "%y.%m.%d"),
        )

    dt = datetime(2021, 1, 1, 0, 0, 0)
    f = Foo(10, dt, dt)

    s = '{"a":10,"b":"01/01/21","c":"21.01.01"}'
    assert s == to_json(f)
    assert f == from_json(Foo, s)

    t = (10, "01/01/21", "21.01.01")
    assert t == to_tuple(f)
    assert f == from_tuple(Foo, t)


def test_custom_serializer_union() -> None:
    class MySerializer(ClassSerializer):
        @dispatch
        def serialize(self, value: datetime) -> str:
            return value.strftime("%d/%m/%y")

    class MyDeserializer(ClassDeserializer):
        @dispatch
        def deserialize(self, cls: type[datetime], value: Any) -> datetime:
            return datetime.strptime(value, "%d/%m/%y")

    @serde(class_serializer=MySerializer(), class_deserializer=MyDeserializer())
    class Bar:
        v: datetime

    @serde(class_serializer=MySerializer(), class_deserializer=MyDeserializer())
    class Baz:
        v: datetime

    @serde(class_serializer=MySerializer(), class_deserializer=MyDeserializer())
    class Foo:
        v: Union[Bar, Baz]

    dt = datetime(2021, 1, 1, 0, 0, 0)
    f = Foo(Baz(dt))
    s = '{"v":{"Baz":{"v":"01/01/21"}}}'
    assert s == to_json(f)
    assert f == from_json(Foo, s)


def test_custom_class_serializer_optional() -> None:
    class MySerializer(ClassSerializer):
        @dispatch
        def serialize(self, value: datetime) -> str:
            return value.strftime("%d/%m/%y")

    class MyDeserializer(ClassDeserializer):
        @dispatch
        def deserialize(self, cls: type[datetime], value: Any) -> datetime:
            return datetime.strptime(value, "%d/%m/%y")

    @serde(class_serializer=MySerializer(), class_deserializer=MyDeserializer())
    class Foo:
        a: datetime
        b: Optional[datetime]
        c: Optional[datetime]

    dt = datetime(2021, 1, 1, 0, 0, 0)
    f = Foo(dt, dt, None)

    s = '{"a":"01/01/21","b":"01/01/21","c":null}'
    assert s == to_json(f)
    assert f == from_json(Foo, s)



"""
Tests for custom serializer/deserializer.
"""

from datetime import datetime
from typing import Any, Optional, Union

import pytest

from serde import (
    SerdeSkip,
    default_deserializer,
    default_serializer,
    field,
    from_tuple,
    serde,
    to_tuple,
    SerdeError,
)
from serde.json import from_json, to_json


def test_legacy_custom_class_serializer() -> None:
    def serializer(cls: Any, o: Any) -> Any:
        if cls is datetime:
            return o.strftime("%d/%m/%y")
        else:
            raise SerdeSkip()

    def deserializer(cls: Any, o: Any) -> Any:
        if cls is datetime:
            return datetime.strptime(o, "%d/%m/%y")
        else:
            raise SerdeSkip()

    @serde(serializer=serializer, deserializer=deserializer)
    class Foo:
        a: int
        b: datetime
        c: datetime
        d: Optional[str] = None
        e: Union[str, int] = 10
        f: list[int] = field(default_factory=list)
        g: set[int] = field(default_factory=set)

    dt = datetime(2021, 1, 1, 0, 0, 0)
    f = Foo(10, dt, dt, f=[1, 2, 3], g={4, 5, 6})

    assert (
        to_json(f)
        == '{"a":10,"b":"01/01/21","c":"01/01/21","d":null,"e":10,"f":[1,2,3],"g":[4,5,6]}'
    )
    assert f == from_json(Foo, to_json(f))

    assert to_tuple(f) == (10, "01/01/21", "01/01/21", None, 10, [1, 2, 3], {4, 5, 6})
    assert f == from_tuple(Foo, to_tuple(f))

    def fallback(_: Any, __: Any) -> Any:
        raise SerdeSkip()

    @serde(serializer=fallback, deserializer=fallback)
    class Foo2:
        a: Optional[str]
        b: str

    f2 = Foo2("foo", "bar")
    assert to_json(f2) == '{"a":"foo","b":"bar"}'
    assert f2 == from_json(Foo2, '{"a":"foo","b":"bar"}')
    assert Foo2(None, "bar") == from_json(Foo2, '{"b":"bar"}')
    with pytest.raises(SerdeError):
        assert Foo2(None, "bar") == from_json(Foo2, "{}")
    with pytest.raises(SerdeError):
        assert Foo2("foo", "bar") == from_json(Foo2, '{"a": "foo"}')


def test_field_serialize_override_legacy_class_serializer() -> None:
    def serializer(cls: Any, o: Any) -> Any:
        if cls is datetime:
            return o.strftime("%d/%m/%y")
        else:
            raise SerdeSkip()

    def deserializer(cls: Any, o: Any) -> Any:
        if cls is datetime:
            return datetime.strptime(o, "%d/%m/%y")
        else:
            raise SerdeSkip()

    @serde(serializer=serializer, deserializer=deserializer)
    class Foo:
        a: int
        b: datetime
        c: datetime = field(
            serializer=lambda x: x.strftime("%y.%m.%d"),
            deserializer=lambda x: datetime.strptime(x, "%y.%m.%d"),
        )

    dt = datetime(2021, 1, 1, 0, 0, 0)
    f = Foo(10, dt, dt)

    assert to_json(f) == '{"a":10,"b":"01/01/21","c":"21.01.01"}'
    assert f == from_json(Foo, to_json(f))

    assert to_tuple(f) == (10, "01/01/21", "21.01.01")
    assert f == from_tuple(Foo, to_tuple(f))


def test_override_by_default_serializer() -> None:
    def serializer(cls: Any, o: Any) -> Any:
        if cls is datetime:
            return o.strftime("%d/%m/%y")
        else:
            raise SerdeSkip()

    def deserializer(cls: Any, o: Any) -> Any:
        if cls is datetime:
            return datetime.strptime(o, "%d/%m/%y")
        else:
            raise SerdeSkip()

    @serde(serializer=serializer, deserializer=deserializer)
    class Foo:
        a: int
        b: datetime
        c: datetime = field(serializer=default_serializer, deserializer=default_deserializer)

    dt = datetime(2021, 1, 1, 0, 0, 0)
    f = Foo(10, dt, dt)

    assert to_json(f) == '{"a":10,"b":"01/01/21","c":"2021-01-01T00:00:00"}'
    assert f == from_json(Foo, to_json(f))

    assert to_tuple(f) == (10, "01/01/21", datetime(2021, 1, 1, 0, 0))
    assert f == from_tuple(Foo, to_tuple(f))



from dataclasses import dataclass, is_dataclass

from serde.core import should_impl_dataclass


def test_should_impl_dataclass() -> None:
    @dataclass
    class Base:
        a: int

    class Derived(Base):
        b: int

    is_dataclass(Base)
    is_dataclass(Derived)
    assert not should_impl_dataclass(Base)
    assert should_impl_dataclass(Derived)

    @dataclass
    class Base2:
        pass

    class Derived2(Base2):
        pass

    is_dataclass(Base2)
    is_dataclass(Derived2)
    assert not should_impl_dataclass(Base2)
    assert not should_impl_dataclass(Derived2)

    @dataclass
    class Base3:
        pass

    class Derived3(Base3):
        b: int

    is_dataclass(Base3)
    is_dataclass(Derived3)
    assert not should_impl_dataclass(Base3)
    assert should_impl_dataclass(Derived3)

    class Base4:
        a: int

    @dataclass
    class Derived4(Base4):
        b: int

    is_dataclass(Base4)
    is_dataclass(Derived4)
    assert should_impl_dataclass(Base4)
    assert not should_impl_dataclass(Derived4)

    class Base5:
        a: int

    class Derived5(Base5):
        b: int

    is_dataclass(Base5)
    is_dataclass(Derived5)
    assert should_impl_dataclass(Base5)
    assert should_impl_dataclass(Derived5)

    @dataclass
    class Base1:
        a: int

    @dataclass
    class Base6:
        a: int

    class Derived6(Base1, Base6):
        c: int

    is_dataclass(Base1)
    is_dataclass(Derived6)
    assert not should_impl_dataclass(Base1)
    assert not should_impl_dataclass(Base6)
    assert should_impl_dataclass(Derived6)



from __future__ import annotations
from dataclasses import dataclass
import enum
from typing import Optional, Any

from serde import field, serde, disabled

from . import imported


@serde
class Inner:
    i: int


@serde
@dataclass(unsafe_hash=True)
class Int:
    """
    Integer.
    """

    i: int

    @staticmethod
    def uncheck_new(value: Any) -> Int:
        """
        Bypass runtime type checker by mutating inner value.
        """
        obj = Int(0)
        obj.i = value
        return obj


@serde(type_check=disabled)
@dataclass(unsafe_hash=True)
class UncheckedInt:
    """
    Integer.
    """

    i: int


@serde
@dataclass(unsafe_hash=True)
class Str:
    """
    String.
    """

    s: str


@serde
@dataclass(unsafe_hash=True)
class Float:
    """
    Float.
    """

    f: float


@serde
@dataclass(unsafe_hash=True)
class Bool:
    """
    Boolean.
    """

    b: bool


@serde
@dataclass(unsafe_hash=True)
class Pri:
    """
    Primitives.
    """

    i: int
    s: str
    f: float
    b: bool


@serde
class PriOpt:
    """
    Optional Primitives.
    """

    i: Optional[int]
    s: Optional[str]
    f: Optional[float]
    b: Optional[bool]


@serde
class PriList:
    """
    List containing primitives.
    """

    i: list[int]
    s: list[str]
    f: list[float]
    b: list[bool]


@serde
class PriDict:
    """
    Dict containing primitives.
    """

    i: dict[str, int]
    s: dict[str, str]
    f: dict[str, float]
    b: dict[str, bool]


@serde
class PriTuple:
    """
    Tuple containing primitives.
    """

    i: tuple[int, int, int]
    s: tuple[str, str, str, str]
    f: tuple[float, float, float, float, float]
    b: tuple[bool, bool, bool, bool, bool, bool]


@dataclass(unsafe_hash=True)
class NestedInt:
    """
    Nested integer.
    """

    i: Int


@serde
@dataclass(unsafe_hash=True)
class NestedPri:
    """
    Nested primitives.
    """

    i: Int
    s: Str
    f: Float
    b: Bool


@serde
class NestedPriOpt:
    """
    Optional Primitives.
    """

    i: Optional[Int]
    s: Optional[Str]
    f: Optional[Float]
    b: Optional[Bool]


@serde
class NestedPriList:
    """
    List containing nested primitives.
    """

    i: list[Int]
    s: list[Str]
    f: list[Float]
    b: list[Bool]


@serde
class NestedPriDict:
    """
    Dict containing nested primitives.
    """

    i: dict[Str, Int]
    s: dict[Str, Str]
    f: dict[Str, Float]
    b: dict[Str, Bool]


@serde
class NestedPriTuple:
    """
    Tuple containing nested primitives.
    """

    i: tuple[Int, Int, Int]
    s: tuple[Str, Str, Str, Str]
    f: tuple[Float, Float, Float, Float, Float]
    b: tuple[Bool, Bool, Bool, Bool, Bool, Bool]


@serde
@dataclass(unsafe_hash=True)
class PriDefault:
    """
    Primitives.
    """

    i: int = 10
    s: str = "foo"
    f: float = 100.0
    b: bool = True


@serde
class OptDefault:
    """
    Optionals.
    """

    n: Optional[int] = None
    i: Optional[int] = 10


class E(enum.Enum):
    S = "foo"
    F = 10.0
    B = True


class IE(enum.IntEnum):
    V0 = enum.auto()
    V1 = enum.auto()
    V2 = 10
    V3 = 100


@serde
class EnumInClass:
    """
    Class having enum fields.
    """

    e: IE = IE.V2
    o: Optional[E] = E.S
    i: imported.IE = imported.IE.V1


@dataclass(unsafe_hash=True)
class Recur:
    a: Optional[Recur]
    b: Optional[list[Recur]]
    c: Optional[dict[str, Recur]]


@dataclass(unsafe_hash=True)
class RecurContainer:
    a: list[RecurContainer]
    b: dict[str, RecurContainer]


serde(Recur)
serde(RecurContainer)


ListPri = list[Pri]

DictPri = dict[str, Pri]

INT = Int(10)

STR = Str("foo")

FLOAT = Float(100.0)

BOOL = Bool(True)

PRI = Pri(10, "foo", 100.0, True)

PRI_TUPLE = (10, "foo", 100.0, True)

PRILIST = ([10], ["foo"], [100.0], [True])

NESTED_PRILIST = ([INT], [STR], [FLOAT], [BOOL])

NESTED_PRILIST_TUPLE = ([(10,)], [("foo",)], [(100.0,)], [(True,)])


@serde
class Init:
    a: int
    b: int = field(init=False)

    def __post_init__(self) -> None:
        self.b = self.a * 10


class StrSubclass(str):
    pass


@serde
class PrimitiveSubclass:
    v: StrSubclass



from __future__ import annotations
from dataclasses import dataclass

from serde import from_dict, serde, to_dict


@dataclass
class Recur:
    f: Recur | None


serde(Recur)


def main() -> None:
    f = Recur(Recur(Recur(None)))
    print(to_dict(f))
    print(from_dict(Recur, {"f": {"f": {"f": None}}}))


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

import sys
import os

sys.path.insert(0, os.path.join(os.path.dirname(__file__), ".."))

from dataclasses import dataclass
import serde
from serde.core import init

# Enable debug mode to see generated code
init(debug=True)


@serde.serde
@dataclass
class Small:
    i: int
    s: str
    f: float
    b: bool


# Create an instance and access the scope to see generated code
small = Small(i=10, s="hello", f=3.14, b=True)
scope = getattr(Small, serde.core.SERDE_SCOPE)
print("=== Generated Serialization Code ===")
for name, code in scope.code.items():
    print(f"\n--- {name} ---")
    print(code)

# Also show what functions are available
print("\n=== Available Functions ===")
for name, func in scope.funcs.items():
    print(f"{name}: {func}")



from serde import coerce, serde
from serde.json import from_json, to_json


@serde(type_check=coerce)
class Bar:
    e: int


@serde(type_check=coerce)
class Foo:
    a: int
    b: list[int]
    c: list[dict[str, int]]
    d: Bar | None = None


def main() -> None:
    f = Foo(a="1", b=[True], c=[{10: 1.0}])  # type: ignore
    print(f"Into Json: {to_json(f)}")

    s = '{"a": "1", "b": [false], "c": [{"10": 1.0}], "d": {"e": "100"}}'
    print(f"From Json: {from_json(Foo, s)}")


if __name__ == "__main__":
    main()



"""
jsonfile.py

Deserialize JSON into an object.

Usage:
    $ uv sync
    $ uv run python jsonfile.py
"""

from serde import serde
from serde.json import from_json


@serde
class Slide:
    title: str
    type: str
    items: list[str] | None


@serde
class Slideshow:
    author: str
    date: str
    slides: list[Slide]
    title: str


@serde
class Data:
    slideshow: Slideshow


def main() -> None:
    text = r"""{
  "slideshow": {
    "author": "Yours Truly",
    "date": "date of publication",
    "slides": [
      {
        "title": "Wake up to WonderWidgets!",
        "type": "all"
      },
      {
        "items": [
          "Why <em>WonderWidgets</em> are great",
          "Who <em>buys</em> WonderWidgets"
        ],
        "title": "Overview",
        "type": "all"
      }
    ],
    "title": "Sample Slide Show"
  }
}
"""
    data = from_json(Data, text)
    print(data)


if __name__ == "__main__":
    main()



import json
from functools import partial
from typing import Any, Union

import attr
import data
from runner import Runner, Size


@attr.s(auto_attribs=True)
class Small:
    i: int
    s: str
    f: float
    b: bool


@attr.s(auto_attribs=True)
class Medium:
    inner: list[Small] = attr.Factory(list)


@attr.s(auto_attribs=True)
class Large:
    customer_id: int
    name: str
    email: str
    preferences: dict[str, Union[str, bool, int]] = attr.Factory(dict)
    items_list: list[str] = attr.Factory(list)
    nested_data: dict[str, list[int]] = attr.Factory(dict)
    loyalty_points: int = 0
    created_at: str = ""


SMALL = Small(**data.args_sm)

MEDIUM = Medium([Small(**d) for d in data.args_md])


# Create Large instance
def create_large_instance() -> Large:
    return Large(
        customer_id=12345,
        name="John Smith",
        email="john@example.com",
        preferences={
            "theme": "dark",
            "notifications": True,
            "language": "en",
            "max_budget": 5000,
            "auto_renew": False,
            "privacy_level": 3,
        },
        items_list=["laptop", "mouse", "keyboard", "monitor", "speakers"] * 20,  # 100 items
        nested_data={
            "category_1": list(range(50)),
            "category_2": list(range(50, 100)),
            "category_3": list(range(100, 150)),
            "category_4": list(range(150, 200)),
            "category_5": list(range(200, 250)),
        },
        loyalty_points=1250,
        created_at="2024-01-15T10:30:00Z",
    )


LARGE = create_large_instance()


def new(size: Size) -> Runner:
    name = "attrs"
    if size == Size.Small:
        unp = SMALL
    elif size == Size.Medium:
        unp = MEDIUM  # type: ignore[assignment]
    elif size == Size.Large:
        unp = LARGE  # type: ignore[assignment]
    return Runner(name, unp, partial(se, unp), None, partial(astuple, unp), partial(asdict, unp))


def se(obj: Union[Small, Medium, Large]) -> str:
    return json.dumps(attr.asdict(obj))


def astuple(obj: Union[Small, Medium, Large]) -> tuple[Any, ...]:
    return attr.astuple(obj)


def asdict(obj: Union[Small, Medium, Large]) -> dict[str, Any]:
    return attr.asdict(obj)



from envclasses import envclass, load_env
from pathlib import Path

from serde import deserialize
from serde.yaml import from_yaml

basedir = Path(__file__).parent


@deserialize
@envclass
class App:
    addr: str
    port: int
    secret: str
    workers: int


def main() -> None:
    with open(basedir / "app.yml") as f:
        yml = f.read()
    cfg = from_yaml(App, yml)
    print(cfg)

    load_env(cfg, prefix="APP")
    print(cfg)


if __name__ == "__main__":
    main()



from serde import disabled, serde
from serde.json import from_json, to_json


@serde(type_check=disabled)
class Foo:
    a: int
    b: list[int]
    c: list[dict[str, int]]


def main() -> None:
    # Foo is instantiated with wrong types but serde won't complain
    f = Foo(a=1.0, b=[1.0], c=[{"k": 1.0}])  # type: ignore
    print(f"Into Json: {to_json(f)}")

    # Also, JSON contains wrong types of values but serde won't complain
    s = '{"a": 1, "b": [1], "c": [{"k": 1.0}]}'
    print(f"From Json: {from_json(Foo, s)}")


if __name__ == "__main__":
    main()



"""
Serialize and Deserialize in YAML format. This module depends on
[pyyaml](https://pypi.org/project/PyYAML/) package.
"""

from typing import overload, Optional, Any

import yaml

from .compat import SerdeError, T
from .de import Deserializer, from_dict
from .se import Serializer, to_dict

__all__ = ["from_yaml", "to_yaml", "deserialize_yaml_numbers"]


class YamlSerializer(Serializer[str]):
    @classmethod
    def serialize(cls, obj: Any, **opts: Any) -> str:
        return yaml.safe_dump(obj, **opts)  # type: ignore


class YamlDeserializer(Deserializer[str]):
    @classmethod
    def deserialize(cls, data: str, **opts: Any) -> Any:
        return yaml.safe_load(data, **opts)


def deserialize_yaml_numbers(value: Any) -> float:
    """
    Convert YAML numbers to float, accepting ints/floats and numeric strings (e.g. scientific
    notation). Rejects booleans and non-numeric inputs.
    """
    if isinstance(value, bool):
        raise SerdeError(f"Expected YAML number but got boolean {value!r}")
    if isinstance(value, (int, float)):
        return float(value)
    if isinstance(value, str):
        try:
            return float(value)
        except ValueError as e:
            raise SerdeError(f"Could not convert YAML number string {value!r} to float") from e
    raise SerdeError(f"Expected YAML number but got {type(value).__name__}")


def to_yaml(
    obj: Any,
    cls: Optional[Any] = None,
    se: type[Serializer[str]] = YamlSerializer,
    reuse_instances: bool = False,
    convert_sets: bool = True,
    skip_none: bool = False,
    **opts: Any,
) -> str:
    """
    Serialize the object into YAML.

    You can pass any serializable `obj`. If you supply keyword arguments other than `se`,
    they will be passed in `yaml.safe_dump` function.

    * `skip_none`: When set to True, any field in the class with a None value is excluded from the
    serialized output. Defaults to False.

    If you want to use the other yaml package, you can subclass `YamlSerializer` and implement
    your own logic.
    """
    return se.serialize(
        to_dict(
            obj,
            c=cls,
            reuse_instances=reuse_instances,
            convert_sets=convert_sets,
            skip_none=skip_none,
        ),
        **opts,
    )


@overload
def from_yaml(
    c: type[T],
    s: str,
    de: type[Deserializer[str]] = YamlDeserializer,
    coerce_numbers: bool = True,
    **opts: Any,
) -> T: ...


# For Union, Optional etc.
@overload
def from_yaml(
    c: Any,
    s: str,
    de: type[Deserializer[str]] = YamlDeserializer,
    coerce_numbers: bool = True,
    **opts: Any,
) -> Any: ...


def from_yaml(
    c: Any,
    s: str,
    de: type[Deserializer[str]] = YamlDeserializer,
    coerce_numbers: bool = True,
    **opts: Any,
) -> Any:
    """
    `c` is a class object and `s` is YAML string. If you supply keyword arguments other than `de`,
    they will be passed in `yaml.safe_load` function.

    * `coerce_numbers`: When True (default), numeric YAML scalars or numeric strings (e.g.
      "1e-3") are coerced to floats when the target type is float. Booleans are rejected.

    If you want to use the other yaml package, you can subclass `YamlDeserializer` and implement
    your own logic.
    """
    deserialize_numbers = deserialize_yaml_numbers if coerce_numbers else None
    return from_dict(
        c,
        de.deserialize(s, **opts),
        reuse_instances=False,
        deserialize_numbers=deserialize_numbers,
    )



# mypy: ignore-errors
from serde import serde
from serde.json import from_json, to_json


# Thanks to PEP681 @dataclass_transform, @dataclass decorator is no longer mandatory
# if you use PEP681 supported type check such as pyright. If you are a mypy user,
# you still need @dataclass decorator.
@serde
class Foo:
    i: int
    s: str
    f: float
    b: bool


def main() -> None:
    f = Foo(10, "foo", 100.0, True)
    print(f"Into Json: {to_json(f)}")

    s = '{"i": 10, "s": "foo", "f": 100.0, "b": true}'
    print(f"From Json: {from_json(Foo, s)}")


if __name__ == "__main__":
    main()



"""
yamlfile.py

Read swagger echo example yaml.

Usage:
    $ uv sync
    $ uv run python yamlfile.py
"""

import pathlib

from serde import Untagged, serde
from serde.yaml import from_yaml

basedir = pathlib.Path(__file__).parent


@serde(rename_all="camelcase")
class Info:
    title: str
    description: str
    version: str


@serde(rename_all="camelcase")
class Parameter:
    name: str
    # not yet supported.
    # infield: str = field(rename='in')
    type: str
    required: bool


@serde(rename_all="camelcase")
class Response:
    description: str


@serde(rename_all="camelcase", tagging=Untagged)
class Path:
    description: str
    operation_id: str
    parameters: list[str | Parameter]
    responses: dict[str | int, Response]


@serde(rename_all="camelcase")
class Prop:
    type: str
    format: str | None


@serde(rename_all="camelcase")
class Definition:
    required: list[str] | None
    properties: dict[str, Prop]


@serde(rename_all="camelcase")
class Swagger:
    swagger: int
    info: Info
    host: str
    schemes: list[str]
    base_path = str
    produces: list[str]
    paths: dict[str, dict[str, Path]]
    definitions: dict[str, Definition]


def main() -> None:
    with open(basedir / "swagger.yml") as f:
        yaml = f.read()
    swagger = from_yaml(Swagger, yaml)
    print(swagger)


if __name__ == "__main__":
    main()



from serde import serde
from serde.msgpack import from_msgpack, to_msgpack


@serde
class Foo:
    i: int
    s: str
    f: float
    b: bool


def main() -> None:
    f = Foo(i=10, s="foo", f=100.0, b=True)
    p = to_msgpack(f)
    print("Into MsgPack:", p)
    print(f"From MsgPack: {from_msgpack(Foo, p)}")


if __name__ == "__main__":
    main()



import json
from functools import partial
from typing import Any

import data
from dataclasses_class import LARGE, MEDIUM, SMALL, Large, Medium, Small
from runner import Runner, Size


def new(size: Size) -> Runner:
    name = "raw"
    if size == Size.Small:
        unp = SMALL
        pac = data.SMALL
        se = se_small
        de = de_small
        astuple = astuple_small
        asdict = asdict_small
    elif size == Size.Medium:
        unp = MEDIUM
        pac = data.MEDIUM
        se = se_medium  # type: ignore[assignment]
        de = de_medium
        astuple = astuple_medium  # type: ignore[assignment]
        asdict = asdict_medium  # type: ignore[assignment]
    elif size == Size.Large:
        unp = LARGE
        pac = data.LARGE
        se = se_large  # type: ignore[assignment]
        de = de_large
        astuple = astuple_large  # type: ignore[assignment]
        asdict = asdict_large  # type: ignore[assignment]
    return Runner(
        name, unp, partial(se, unp), partial(de, pac), partial(astuple, unp), partial(asdict, unp)
    )


def se_small(s: Small) -> str:
    return json.dumps(asdict_small(s))


def se_medium(m: Medium) -> str:
    return json.dumps({"inner": [{"i": s.i, "s": s.s, "f": s.f, "b": s.b} for s in m.inner]})


def de_small(data: str) -> Small:
    return _de_small(json.loads(data))


def _de_small(dct: dict[str, Any]) -> Small:
    return Small(dct["i"], dct["s"], dct["f"], dct["b"])


def de_medium(data: str) -> Medium:
    lst = json.loads(data)
    return Medium([_de_small(v) for v in lst["inner"]])


def astuple_small(sm: Small) -> tuple[int, str, float, bool]:
    return (sm.i, sm.s, sm.f, sm.b)


def astuple_medium(md: Medium) -> tuple[list[tuple[int, str, float, bool]]]:
    return ([astuple_small(sm) for sm in md.inner],)


def asdict_small(s: Small) -> dict[str, Any]:
    return {"i": s.i, "s": s.s, "f": s.f, "b": s.b}


def asdict_medium(m: Medium) -> dict[str, Any]:
    return {"inner": [asdict_small(s) for s in m.inner]}


def se_large(large: Large) -> str:
    return json.dumps(asdict_large(large))


def de_large(data: str) -> Large:
    return _de_large(json.loads(data))


def _de_large(dct: dict[str, Any]) -> Large:
    return Large(
        customer_id=dct["customer_id"],
        name=dct["name"],
        email=dct["email"],
        preferences=dct["preferences"],
        items_list=dct["items_list"],
        nested_data=dct["nested_data"],
        loyalty_points=dct["loyalty_points"],
        created_at=dct["created_at"],
    )


def astuple_large(large: Large) -> tuple[Any, ...]:
    # Complex tuple conversion for large nested structure
    from dataclasses import astuple

    return astuple(large)


def asdict_large(large: Large) -> dict[str, Any]:
    # Complex dict conversion for large nested structure
    from dataclasses import asdict

    return asdict(large)



from serde import serde, serialize, deserialize
from serde.json import from_json, to_json
from dataclasses import dataclass
from beartype import beartype
import cProfile
import pstats


@beartype
@dataclass
class FewFields:
    a: int
    b: float
    c: str
    d: bool


@beartype
@dataclass
class ManyFields:
    a1: int
    a2: int
    a3: int
    a4: int
    a5: int
    a6: int
    a7: int
    a8: int
    a9: int
    a10: int
    a11: int
    a12: int
    a13: int
    a14: int
    a15: int
    a16: int
    a17: int
    a18: int
    a19: int
    a20: int
    b1: int
    b2: int
    b3: int
    b4: int
    b5: int
    b6: int
    b7: int
    b8: int
    b9: int
    b10: int
    b11: int
    b12: int
    b13: int
    b14: int
    b15: int
    b16: int
    b17: int
    b18: int
    b19: int
    b20: int
    c1: int
    c2: int
    c3: int
    c4: int
    c5: int
    c6: int
    c7: int
    c8: int
    c9: int
    c10: int
    c11: int
    c12: int
    c13: int
    c14: int
    c15: int
    c16: int
    c17: int
    c18: int
    c19: int
    c20: int
    d1: int
    d2: int
    d3: int
    d4: int
    d5: int
    d6: int
    d7: int
    d8: int
    d9: int
    d10: int
    d11: int
    d12: int
    d13: int
    d14: int
    d15: int
    d16: int
    d17: int
    d18: int
    d19: int
    d20: int


def profile_few_fields() -> None:
    for n in range(100):
        serde(FewFields)


def profile_many_fields() -> None:
    for n in range(100):
        serde(ManyFields)


cProfile.run("profile_few_fields()", filename="profile_results.prof")
stats = pstats.Stats("profile_results.prof")
stats.sort_stats("tottime").print_stats(20)

cProfile.run("profile_many_fields()", filename="profile_results.prof")
stats = pstats.Stats("profile_results.prof")
stats.sort_stats("tottime").print_stats(20)



import numpy

from serde import serde
from serde.json import from_json, to_json


@serde
class Foo:
    bo: numpy.bool_
    by: numpy.byte
    in_: numpy.int_
    inc: numpy.intc
    ui: numpy.uint
    fl: numpy.float64
    st: numpy.str_
    nd: numpy.typing.NDArray[numpy.int_]
    ha: numpy.half
    da: numpy.datetime64


def main() -> None:
    foo = Foo(
        bo=numpy.bool_(True),
        by=numpy.byte(38),
        in_=numpy.int_(42),
        inc=numpy.intc(42),
        ui=numpy.uint(42),
        fl=numpy.float64(3.14),
        st=numpy.str_("numpy str"),
        nd=numpy.array([1, 2, 3]),
        ha=numpy.half(3.14),
        da=numpy.datetime64("2020-01-01T03:30:00.162"),
    )
    print(f"Into Json: {to_json(foo)}")

    s = """{
        "bo": true,
        "by": 38,
        "in_": 42,
        "inc": 42,
        "ui": 42,
        "fl": 3.14,
        "st": "numpy str",
        "nd": [1, 2, 3],
        "ha": 3.14,
        "da": "2020-01-01T03:30:00.162"
    }"""
    print(f"From Json: {from_json(Foo, s)}")


if __name__ == "__main__":
    main()



from typing import Optional

from sqlalchemy import Text, JSON, ForeignKey
from sqlalchemy.orm import DeclarativeBase, MappedAsDataclass, Mapped, mapped_column, relationship

from serde import serde
from serde.json import from_json, to_json


class Base(MappedAsDataclass, DeclarativeBase):  # type: ignore[misc]
    pass


@serde
class Project(Base):
    __tablename__ = "projects"
    id: Mapped[int] = mapped_column(primary_key=True)
    name: Mapped[str] = mapped_column(Text, nullable=False)
    owner_id: Mapped[int] = mapped_column(ForeignKey("users.id"))


@serde
class User(Base):
    __tablename__ = "users"

    id: Mapped[int] = mapped_column(primary_key=True)
    name: Mapped[str] = mapped_column(Text, nullable=False)
    fullname: Mapped[str] = mapped_column(Text, nullable=False)
    nickname: Mapped[Optional[str]] = mapped_column(Text)
    attributes: Mapped[Optional[dict[str, str]]] = mapped_column(JSON)
    projects: Mapped[list[Project]] = relationship(backref="owner")


def main() -> None:
    user = User(
        id=1,
        name="john",
        fullname="John Doe",
        nickname=None,
        attributes={"color": "green"},
        projects=[Project(id=1, name="Dummy", owner_id=1)],
    )
    print(f"Into Json: {to_json(user)}")

    s = """{
        "id": 1,
        "name": "john",
        "fullname": "John Doe",
        "nickname": null,
        "attributes": {
            "color": "green"
        },
        "projects": [
            {
                "id": 1,
                "name": "Dummy",
                "owner_id": 1
            }
        ]
    }"""
    print(f"From Json: {from_json(User, s)}")


if __name__ == "__main__":
    main()
